from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.exb_declarations import ExbDeclarations
from ..models.exb_employer import ExbEmployer
from ..models.exb_p11_db import ExbP11Db
from ..models.exb_p11d import ExbP11D
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExpensesAndBenefits")


@attr.s(auto_attribs=True)
class ExpensesAndBenefits:
    """
    Attributes:
        employer (Union[Unset, ExbEmployer]):
        declarations (Union[Unset, ExbDeclarations]):
        p_11_db (Union[Unset, ExbP11Db]):
        p_11_drecord_count (Union[Unset, int]):
        p_46_car_record_count (Union[Unset, int]):
        p_11d (Union[Unset, None, List[ExbP11D]]):
        related_tax_year (Union[Unset, None, str]):
    """

    employer: Union[Unset, ExbEmployer] = UNSET
    declarations: Union[Unset, ExbDeclarations] = UNSET
    p_11_db: Union[Unset, ExbP11Db] = UNSET
    p_11_drecord_count: Union[Unset, int] = UNSET
    p_46_car_record_count: Union[Unset, int] = UNSET
    p_11d: Union[Unset, None, List[ExbP11D]] = UNSET
    related_tax_year: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer, Unset):
            employer = self.employer.to_dict()

        declarations: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.declarations, Unset):
            declarations = self.declarations.to_dict()

        p_11_db: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.p_11_db, Unset):
            p_11_db = self.p_11_db.to_dict()

        p_11_drecord_count = self.p_11_drecord_count
        p_46_car_record_count = self.p_46_car_record_count
        p_11d: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.p_11d, Unset):
            if self.p_11d is None:
                p_11d = None
            else:
                p_11d = []
                for p_11d_item_data in self.p_11d:
                    p_11d_item = p_11d_item_data.to_dict()

                    p_11d.append(p_11d_item)

        related_tax_year = self.related_tax_year

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employer is not UNSET:
            field_dict["employer"] = employer
        if declarations is not UNSET:
            field_dict["declarations"] = declarations
        if p_11_db is not UNSET:
            field_dict["p11Db"] = p_11_db
        if p_11_drecord_count is not UNSET:
            field_dict["p11DrecordCount"] = p_11_drecord_count
        if p_46_car_record_count is not UNSET:
            field_dict["p46CarRecordCount"] = p_46_car_record_count
        if p_11d is not UNSET:
            field_dict["p11D"] = p_11d
        if related_tax_year is not UNSET:
            field_dict["relatedTaxYear"] = related_tax_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employer = d.pop("employer", UNSET)
        employer: Union[Unset, ExbEmployer]
        if isinstance(_employer, Unset):
            employer = UNSET
        else:
            employer = ExbEmployer.from_dict(_employer)

        _declarations = d.pop("declarations", UNSET)
        declarations: Union[Unset, ExbDeclarations]
        if isinstance(_declarations, Unset):
            declarations = UNSET
        else:
            declarations = ExbDeclarations.from_dict(_declarations)

        _p_11_db = d.pop("p11Db", UNSET)
        p_11_db: Union[Unset, ExbP11Db]
        if isinstance(_p_11_db, Unset):
            p_11_db = UNSET
        else:
            p_11_db = ExbP11Db.from_dict(_p_11_db)

        p_11_drecord_count = d.pop("p11DrecordCount", UNSET)

        p_46_car_record_count = d.pop("p46CarRecordCount", UNSET)

        p_11d = []
        _p_11d = d.pop("p11D", UNSET)
        for p_11d_item_data in _p_11d or []:
            p_11d_item = ExbP11D.from_dict(p_11d_item_data)

            p_11d.append(p_11d_item)

        related_tax_year = d.pop("relatedTaxYear", UNSET)

        expenses_and_benefits = cls(
            employer=employer,
            declarations=declarations,
            p_11_db=p_11_db,
            p_11_drecord_count=p_11_drecord_count,
            p_46_car_record_count=p_46_car_record_count,
            p_11d=p_11d,
            related_tax_year=related_tax_year,
        )

        return expenses_and_benefits



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.p11_detailed import P11Detailed
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DetailedReportResponse")


@attr.s(auto_attribs=True)
class P11DetailedReportResponse:
    """Used to encapsulate a response for any of the reports.
    See the Introduction Guide for Reports for more details

        Attributes:
            type (Union[Unset, None, str]): [readonly] The content-type, this would usually be the same as the accept header
                you provided when you requested the report
            content (Union[Unset, None, str]): [readonly] This could contain a link to a PDF file, HTML content or other
                content, depending on the Type value.
            model (Union[Unset, P11Detailed]): A more comprehensive P11 Report.
                Our Reports API can return this to you in several formats including as a PDF file
                If you request a JSON object then it is represented using this model.
    """

    type: Union[Unset, None, str] = UNSET
    content: Union[Unset, None, str] = UNSET
    model: Union[Unset, P11Detailed] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        content = self.content
        model: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.model, Unset):
            model = self.model.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if content is not UNSET:
            field_dict["content"] = content
        if model is not UNSET:
            field_dict["model"] = model

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        content = d.pop("content", UNSET)

        _model = d.pop("model", UNSET)
        model: Union[Unset, P11Detailed]
        if isinstance(_model, Unset):
            model = UNSET
        else:
            model = P11Detailed.from_dict(_model)

        p11_detailed_report_response = cls(
            type=type,
            content=content,
            model=model,
        )

        return p11_detailed_report_response



import datetime
from typing import Any, Dict, Optional, Union, cast

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    tax_code: Union[Unset, None, str] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
    period: Union[Unset, None, int] = 1,
    week1: Union[Unset, None, bool] = False,
    gross_to_date: Union[Unset, None, float] = 0.0,
    tax_to_date: Union[Unset, None, float] = 0.0,
) -> Dict[str, Any]:
    url = "{}/engine/tax".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_tax_year: Union[Unset, None, str] = UNSET
    if not isinstance(tax_year, Unset):
        json_tax_year = tax_year.value if tax_year else None

    params["taxYear"] = json_tax_year

    params["taxCode"] = tax_code

    params["gross"] = gross

    json_period_start: Union[Unset, None, str] = UNSET
    if not isinstance(period_start, Unset):
        json_period_start = period_start.isoformat() if period_start else None

    params["periodStart"] = json_period_start

    json_period_end: Union[Unset, None, str] = UNSET
    if not isinstance(period_end, Unset):
        json_period_end = period_end.isoformat() if period_end else None

    params["periodEnd"] = json_period_end

    json_pay_period: Union[Unset, None, str] = UNSET
    if not isinstance(pay_period, Unset):
        json_pay_period = pay_period.value if pay_period else None

    params["payPeriod"] = json_pay_period

    params["period"] = period

    params["week1"] = week1

    params["grossToDate"] = gross_to_date

    params["taxToDate"] = tax_to_date

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[float]:
    if response.status_code == 200:
        response_200 = cast(float, response.json())
        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[float]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    tax_code: Union[Unset, None, str] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
    period: Union[Unset, None, int] = 1,
    week1: Union[Unset, None, bool] = False,
    gross_to_date: Union[Unset, None, float] = 0.0,
    tax_to_date: Union[Unset, None, float] = 0.0,
) -> Response[float]:
    """Calculate Tax due

     Calculates tax amount due, given the values specified.
    You would never need to use this API call. It is provided just for information and testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        tax_code (Union[Unset, None, str]):
        gross (Union[Unset, None, float]):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        pay_period (Union[Unset, None, PayPeriods]):
        period (Union[Unset, None, int]):  Default: 1.
        week1 (Union[Unset, None, bool]):
        gross_to_date (Union[Unset, None, float]):
        tax_to_date (Union[Unset, None, float]):

    Returns:
        Response[float]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
        tax_code=tax_code,
        gross=gross,
        period_start=period_start,
        period_end=period_end,
        pay_period=pay_period,
        period=period,
        week1=week1,
        gross_to_date=gross_to_date,
        tax_to_date=tax_to_date,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    tax_code: Union[Unset, None, str] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
    period: Union[Unset, None, int] = 1,
    week1: Union[Unset, None, bool] = False,
    gross_to_date: Union[Unset, None, float] = 0.0,
    tax_to_date: Union[Unset, None, float] = 0.0,
) -> Optional[float]:
    """Calculate Tax due

     Calculates tax amount due, given the values specified.
    You would never need to use this API call. It is provided just for information and testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        tax_code (Union[Unset, None, str]):
        gross (Union[Unset, None, float]):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        pay_period (Union[Unset, None, PayPeriods]):
        period (Union[Unset, None, int]):  Default: 1.
        week1 (Union[Unset, None, bool]):
        gross_to_date (Union[Unset, None, float]):
        tax_to_date (Union[Unset, None, float]):

    Returns:
        Response[float]
    """

    return sync_detailed(
        client=client,
        tax_year=tax_year,
        tax_code=tax_code,
        gross=gross,
        period_start=period_start,
        period_end=period_end,
        pay_period=pay_period,
        period=period,
        week1=week1,
        gross_to_date=gross_to_date,
        tax_to_date=tax_to_date,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    tax_code: Union[Unset, None, str] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
    period: Union[Unset, None, int] = 1,
    week1: Union[Unset, None, bool] = False,
    gross_to_date: Union[Unset, None, float] = 0.0,
    tax_to_date: Union[Unset, None, float] = 0.0,
) -> Response[float]:
    """Calculate Tax due

     Calculates tax amount due, given the values specified.
    You would never need to use this API call. It is provided just for information and testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        tax_code (Union[Unset, None, str]):
        gross (Union[Unset, None, float]):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        pay_period (Union[Unset, None, PayPeriods]):
        period (Union[Unset, None, int]):  Default: 1.
        week1 (Union[Unset, None, bool]):
        gross_to_date (Union[Unset, None, float]):
        tax_to_date (Union[Unset, None, float]):

    Returns:
        Response[float]
    """

    kwargs = _get_kwargs(
        client=client,
        tax_year=tax_year,
        tax_code=tax_code,
        gross=gross,
        period_start=period_start,
        period_end=period_end,
        pay_period=pay_period,
        period=period,
        week1=week1,
        gross_to_date=gross_to_date,
        tax_to_date=tax_to_date,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    tax_year: Union[Unset, None, TaxYear] = UNSET,
    tax_code: Union[Unset, None, str] = UNSET,
    gross: Union[Unset, None, float] = UNSET,
    period_start: Union[Unset, None, datetime.datetime] = UNSET,
    period_end: Union[Unset, None, datetime.datetime] = UNSET,
    pay_period: Union[Unset, None, PayPeriods] = UNSET,
    period: Union[Unset, None, int] = 1,
    week1: Union[Unset, None, bool] = False,
    gross_to_date: Union[Unset, None, float] = 0.0,
    tax_to_date: Union[Unset, None, float] = 0.0,
) -> Optional[float]:
    """Calculate Tax due

     Calculates tax amount due, given the values specified.
    You would never need to use this API call. It is provided just for information and testing purposes.
    Access is limited so you'll probably receive a 401 response if you try to use it.

    Args:
        tax_year (Union[Unset, None, TaxYear]):
        tax_code (Union[Unset, None, str]):
        gross (Union[Unset, None, float]):
        period_start (Union[Unset, None, datetime.datetime]):
        period_end (Union[Unset, None, datetime.datetime]):
        pay_period (Union[Unset, None, PayPeriods]):
        period (Union[Unset, None, int]):  Default: 1.
        week1 (Union[Unset, None, bool]):
        gross_to_date (Union[Unset, None, float]):
        tax_to_date (Union[Unset, None, float]):

    Returns:
        Response[float]
    """

    return (
        await asyncio_detailed(
            client=client,
            tax_year=tax_year,
            tax_code=tax_code,
            gross=gross,
            period_start=period_start,
            period_end=period_end,
            pay_period=pay_period,
            period=period,
            week1=week1,
            gross_to_date=gross_to_date,
            tax_to_date=tax_to_date,
        )
    ).parsed



from typing import Any, Dict, Optional

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.employee import Employee
from ...models.update_photo_employee_multipart_data import (
    UpdatePhotoEmployeeMultipartData,
)
from ...types import Response


def _get_kwargs(
    employer_id: str,
    id: str,
    *,
    client: Client,
    multipart_data: UpdatePhotoEmployeeMultipartData,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/employees/{id}/photo".format(
        client.base_url, employerId=employer_id, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    multipart_multipart_data = multipart_data.to_multipart()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "files": multipart_multipart_data,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Employee]:
    if response.status_code == 200:
        response_200 = Employee.from_dict(response.json())

        return response_200
    return raise_staffology_exception(response)


def _build_response(*, response: httpx.Response) -> Response[Employee]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    id: str,
    *,
    client: Client,
    multipart_data: UpdatePhotoEmployeeMultipartData,
) -> Response[Employee]:
    """Update Photo

     If you already have a URL for the employee photo then you can just set the PhotoUrl property of the
    Employee.
    Alternatively, submit an image here and we'll upload it, resize it to 200px squared and set the
    PhotoUrl for you.

    Args:
        employer_id (str):
        id (str):
        multipart_data (UpdatePhotoEmployeeMultipartData):

    Returns:
        Response[Employee]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        multipart_data=multipart_data,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    id: str,
    *,
    client: Client,
    multipart_data: UpdatePhotoEmployeeMultipartData,
) -> Optional[Employee]:
    """Update Photo

     If you already have a URL for the employee photo then you can just set the PhotoUrl property of the
    Employee.
    Alternatively, submit an image here and we'll upload it, resize it to 200px squared and set the
    PhotoUrl for you.

    Args:
        employer_id (str):
        id (str):
        multipart_data (UpdatePhotoEmployeeMultipartData):

    Returns:
        Response[Employee]
    """

    return sync_detailed(
        employer_id=employer_id,
        id=id,
        client=client,
        multipart_data=multipart_data,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    id: str,
    *,
    client: Client,
    multipart_data: UpdatePhotoEmployeeMultipartData,
) -> Response[Employee]:
    """Update Photo

     If you already have a URL for the employee photo then you can just set the PhotoUrl property of the
    Employee.
    Alternatively, submit an image here and we'll upload it, resize it to 200px squared and set the
    PhotoUrl for you.

    Args:
        employer_id (str):
        id (str):
        multipart_data (UpdatePhotoEmployeeMultipartData):

    Returns:
        Response[Employee]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        multipart_data=multipart_data,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    id: str,
    *,
    client: Client,
    multipart_data: UpdatePhotoEmployeeMultipartData,
) -> Optional[Employee]:
    """Update Photo

     If you already have a URL for the employee photo then you can just set the PhotoUrl property of the
    Employee.
    Alternatively, submit an image here and we'll upload it, resize it to 200px squared and set the
    PhotoUrl for you.

    Args:
        employer_id (str):
        id (str):
        multipart_data (UpdatePhotoEmployeeMultipartData):

    Returns:
        Response[Employee]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            id=id,
            client=client,
            multipart_data=multipart_data,
        )
    ).parsed



from enum import Enum


class UserstartPage(str, Enum):
    DASHBOARD = "Dashboard"
    EMPLOYERLIST = "EmployerList"
    EMPLOYEELIST = "EmployeeList"
    PAYROLL = "Payroll"
    RTI = "Rti"
    BUREAUDASHBOARD = "BureauDashboard"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class BenefitType(str, Enum):
    ASSETSTRANSFERRED = "AssetsTransferred"
    PAYMENTSONBEHALF = "PaymentsOnBehalf"
    UNBORNETAX = "UnborneTax"
    VOUCHERS = "Vouchers"
    ACCOMMODATION = "Accommodation"
    MILEAGEALLOWANCE = "MileageAllowance"
    CAR = "Car"
    VANS = "Vans"
    LOAN = "Loan"
    MEDICAL = "Medical"
    QUALIFYINGRELOCATIONEXPENSES = "QualifyingRelocationExpenses"
    SERVICES = "Services"
    ASSETSATEMPLOYEEDISPOSAL = "AssetsAtEmployeeDisposal"
    OTHERCLASS1AITEMS = "OtherClass1AItems"
    OTHERNONCLASS1AITEMS = "OtherNonClass1AItems"
    DIRECTORTAX = "DirectorTax"
    TRAVELLINGANDSUBSISTENCE = "TravellingAndSubsistence"
    ENTERTAINMENT = "Entertainment"
    HOMETELEPHONE = "HomeTelephone"
    NONQUALIFYINGRELOCATIONEXPENSES = "NonQualifyingRelocationExpenses"
    OTHEREXPENSES = "OtherExpenses"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class HolidayType(str, Enum):
    DAYS = "Days"
    ACCRUAL_MONEY = "Accrual_Money"
    ACCRUAL_DAYS = "Accrual_Days"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PensionCsvFormat(str, Enum):
    PAPDIS = "Papdis"
    NEST = "Nest"
    NOWPENSIONS = "NowPensions"
    TEACHERSPENSIONMDC = "TeachersPensionMdc"
    TEACHERSPENSIONMCR = "TeachersPensionMcr"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.papdis_employee_assessment import PapdisEmployeeAssessment
from ..models.papdis_employee_contact import PapdisEmployeeContact
from ..models.papdis_employee_contribution import PapdisEmployeeContribution
from ..models.papdis_employee_exit import PapdisEmployeeExit
from ..models.papdis_employee_identity import PapdisEmployeeIdentity
from ..models.papdis_employee_name import PapdisEmployeeName
from ..models.papdis_employee_pay import PapdisEmployeePay
from ..types import UNSET, Unset

T = TypeVar("T", bound="PapdisEmployee")


@attr.s(auto_attribs=True)
class PapdisEmployee:
    """
    Attributes:
        employee_id (Union[Unset, str]): [readonly]
        name (Union[Unset, PapdisEmployeeName]):
        identity (Union[Unset, PapdisEmployeeIdentity]):
        contact (Union[Unset, PapdisEmployeeContact]):
        pay (Union[Unset, PapdisEmployeePay]):
        assessment (Union[Unset, PapdisEmployeeAssessment]):
        contribution (Union[Unset, PapdisEmployeeContribution]):
        exit_ (Union[Unset, PapdisEmployeeExit]):
    """

    employee_id: Union[Unset, str] = UNSET
    name: Union[Unset, PapdisEmployeeName] = UNSET
    identity: Union[Unset, PapdisEmployeeIdentity] = UNSET
    contact: Union[Unset, PapdisEmployeeContact] = UNSET
    pay: Union[Unset, PapdisEmployeePay] = UNSET
    assessment: Union[Unset, PapdisEmployeeAssessment] = UNSET
    contribution: Union[Unset, PapdisEmployeeContribution] = UNSET
    exit_: Union[Unset, PapdisEmployeeExit] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_id = self.employee_id
        name: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.name, Unset):
            name = self.name.to_dict()

        identity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.identity, Unset):
            identity = self.identity.to_dict()

        contact: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contact, Unset):
            contact = self.contact.to_dict()

        pay: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pay, Unset):
            pay = self.pay.to_dict()

        assessment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assessment, Unset):
            assessment = self.assessment.to_dict()

        contribution: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contribution, Unset):
            contribution = self.contribution.to_dict()

        exit_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.exit_, Unset):
            exit_ = self.exit_.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_id is not UNSET:
            field_dict["employeeId"] = employee_id
        if name is not UNSET:
            field_dict["name"] = name
        if identity is not UNSET:
            field_dict["identity"] = identity
        if contact is not UNSET:
            field_dict["contact"] = contact
        if pay is not UNSET:
            field_dict["pay"] = pay
        if assessment is not UNSET:
            field_dict["assessment"] = assessment
        if contribution is not UNSET:
            field_dict["contribution"] = contribution
        if exit_ is not UNSET:
            field_dict["exit"] = exit_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_id = d.pop("employeeId", UNSET)

        _name = d.pop("name", UNSET)
        name: Union[Unset, PapdisEmployeeName]
        if isinstance(_name, Unset):
            name = UNSET
        else:
            name = PapdisEmployeeName.from_dict(_name)

        _identity = d.pop("identity", UNSET)
        identity: Union[Unset, PapdisEmployeeIdentity]
        if isinstance(_identity, Unset):
            identity = UNSET
        else:
            identity = PapdisEmployeeIdentity.from_dict(_identity)

        _contact = d.pop("contact", UNSET)
        contact: Union[Unset, PapdisEmployeeContact]
        if isinstance(_contact, Unset):
            contact = UNSET
        else:
            contact = PapdisEmployeeContact.from_dict(_contact)

        _pay = d.pop("pay", UNSET)
        pay: Union[Unset, PapdisEmployeePay]
        if isinstance(_pay, Unset):
            pay = UNSET
        else:
            pay = PapdisEmployeePay.from_dict(_pay)

        _assessment = d.pop("assessment", UNSET)
        assessment: Union[Unset, PapdisEmployeeAssessment]
        if isinstance(_assessment, Unset):
            assessment = UNSET
        else:
            assessment = PapdisEmployeeAssessment.from_dict(_assessment)

        _contribution = d.pop("contribution", UNSET)
        contribution: Union[Unset, PapdisEmployeeContribution]
        if isinstance(_contribution, Unset):
            contribution = UNSET
        else:
            contribution = PapdisEmployeeContribution.from_dict(_contribution)

        _exit_ = d.pop("exit", UNSET)
        exit_: Union[Unset, PapdisEmployeeExit]
        if isinstance(_exit_, Unset):
            exit_ = UNSET
        else:
            exit_ = PapdisEmployeeExit.from_dict(_exit_)

        papdis_employee = cls(
            employee_id=employee_id,
            name=name,
            identity=identity,
            contact=contact,
            pay=pay,
            assessment=assessment,
            contribution=contribution,
            exit_=exit_,
        )

        return papdis_employee



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.payroll_value_type import PayrollValueType
from ..types import UNSET, Unset

T = TypeVar("T", bound="YtdValue")


@attr.s(auto_attribs=True)
class YtdValue:
    """
    Attributes:
        type (Union[Unset, PayrollValueType]):
        brought_forward (Union[Unset, float]):
        period (Union[Unset, float]):
        value (Union[Unset, float]):
    """

    type: Union[Unset, PayrollValueType] = UNSET
    brought_forward: Union[Unset, float] = UNSET
    period: Union[Unset, float] = UNSET
    value: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        brought_forward = self.brought_forward
        period = self.period
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if brought_forward is not UNSET:
            field_dict["broughtForward"] = brought_forward
        if period is not UNSET:
            field_dict["period"] = period
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _type = d.pop("type", UNSET)
        type: Union[Unset, PayrollValueType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = PayrollValueType(_type)

        brought_forward = d.pop("broughtForward", UNSET)

        period = d.pop("period", UNSET)

        value = d.pop("value", UNSET)

        ytd_value = cls(
            type=type,
            brought_forward=brought_forward,
            period=period,
            value=value,
        )

        return ytd_value



from enum import Enum


class PayPeriodEventType(str, Enum):
    SUBMITFORPROCESSING = "SubmitForProcessing"
    SENDFORAPPROVAL = "SendForApproval"
    APPROVAL = "Approval"
    FINALISE = "Finalise"
    SENDPAYSLIP = "SendPaySlip"
    SUBMITRTI = "SubmitRti"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerSettings")


@attr.s(auto_attribs=True)
class EmployerSettings:
    """Miscellaneous settings related to the employer that don't naturally belong in other models

    Attributes:
        allow_negative_pay (Union[Unset, bool]):
    """

    allow_negative_pay: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        allow_negative_pay = self.allow_negative_pay

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if allow_negative_pay is not UNSET:
            field_dict["allowNegativePay"] = allow_negative_pay

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_negative_pay = d.pop("allowNegativePay", UNSET)

        employer_settings = cls(
            allow_negative_pay=allow_negative_pay,
        )

        return employer_settings



from typing import Any, Dict, List, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.external_data_provider_id import ExternalDataProviderId
from ...models.string_string_key_value_pair import StringStringKeyValuePair
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
    json_body: List[StringStringKeyValuePair],
    username: Union[Unset, None, str] = UNSET,
    password: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/external-data/{id}/authorize".format(
        client.base_url, employerId=employer_id, id=id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["username"] = username

    params["password"] = password

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for json_body_item_data in json_body:
        json_body_item = json_body_item_data.to_dict()

        json_json_body.append(json_body_item)

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
    json_body: List[StringStringKeyValuePair],
    username: Union[Unset, None, str] = UNSET,
    password: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Set Credentials

     For ExternalDataProviders with an AuthScheme of Basic.
    Sets the username and password for the service.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):
        username (Union[Unset, None, str]):
        password (Union[Unset, None, str]):
        json_body (List[StringStringKeyValuePair]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        json_body=json_body,
        username=username,
        password=password,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    employer_id: str,
    id: ExternalDataProviderId,
    *,
    client: Client,
    json_body: List[StringStringKeyValuePair],
    username: Union[Unset, None, str] = UNSET,
    password: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Set Credentials

     For ExternalDataProviders with an AuthScheme of Basic.
    Sets the username and password for the service.

    Args:
        employer_id (str):
        id (ExternalDataProviderId):
        username (Union[Unset, None, str]):
        password (Union[Unset, None, str]):
        json_body (List[StringStringKeyValuePair]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        id=id,
        client=client,
        json_body=json_body,
        username=username,
        password=password,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.emp_refs import EmpRefs
from ..models.expenses_and_benefits import ExpensesAndBenefits
from ..models.gov_talk_submission import GovTalkSubmission
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="Exb")


@attr.s(auto_attribs=True)
class Exb:
    """
    Attributes:
        additions_amount (Union[Unset, float]):
        additions_description (Union[Unset, None, str]):
        deductions_amount (Union[Unset, float]):
        deductions_description (Union[Unset, None, str]):
        employee_count (Union[Unset, int]): [readonly] The number of employees with a P11D
        total_benefits (Union[Unset, float]): [readonly] Total value of benefits, not including any deductions or
            additions
        total_adjusted_benefits (Union[Unset, float]): [readonly] Total value of benefits
        nics_rate (Union[Unset, float]): [readonly] The Class 1A NICS Rate
        nics_payable (Union[Unset, float]): [readonly] NICs Payable
        expenses_and_benefits (Union[Unset, ExpensesAndBenefits]):
        i_rmark (Union[Unset, None, str]):
        xml (Union[Unset, None, str]):
        tax_year (Union[Unset, TaxYear]):
        employer_references (Union[Unset, EmpRefs]):
        gov_talk_submission (Union[Unset, GovTalkSubmission]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    additions_amount: Union[Unset, float] = UNSET
    additions_description: Union[Unset, None, str] = UNSET
    deductions_amount: Union[Unset, float] = UNSET
    deductions_description: Union[Unset, None, str] = UNSET
    employee_count: Union[Unset, int] = UNSET
    total_benefits: Union[Unset, float] = UNSET
    total_adjusted_benefits: Union[Unset, float] = UNSET
    nics_rate: Union[Unset, float] = UNSET
    nics_payable: Union[Unset, float] = UNSET
    expenses_and_benefits: Union[Unset, ExpensesAndBenefits] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    xml: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    employer_references: Union[Unset, EmpRefs] = UNSET
    gov_talk_submission: Union[Unset, GovTalkSubmission] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        additions_amount = self.additions_amount
        additions_description = self.additions_description
        deductions_amount = self.deductions_amount
        deductions_description = self.deductions_description
        employee_count = self.employee_count
        total_benefits = self.total_benefits
        total_adjusted_benefits = self.total_adjusted_benefits
        nics_rate = self.nics_rate
        nics_payable = self.nics_payable
        expenses_and_benefits: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.expenses_and_benefits, Unset):
            expenses_and_benefits = self.expenses_and_benefits.to_dict()

        i_rmark = self.i_rmark
        xml = self.xml
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        employer_references: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_references, Unset):
            employer_references = self.employer_references.to_dict()

        gov_talk_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gov_talk_submission, Unset):
            gov_talk_submission = self.gov_talk_submission.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if additions_amount is not UNSET:
            field_dict["additionsAmount"] = additions_amount
        if additions_description is not UNSET:
            field_dict["additionsDescription"] = additions_description
        if deductions_amount is not UNSET:
            field_dict["deductionsAmount"] = deductions_amount
        if deductions_description is not UNSET:
            field_dict["deductionsDescription"] = deductions_description
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if total_benefits is not UNSET:
            field_dict["totalBenefits"] = total_benefits
        if total_adjusted_benefits is not UNSET:
            field_dict["totalAdjustedBenefits"] = total_adjusted_benefits
        if nics_rate is not UNSET:
            field_dict["nicsRate"] = nics_rate
        if nics_payable is not UNSET:
            field_dict["nicsPayable"] = nics_payable
        if expenses_and_benefits is not UNSET:
            field_dict["expensesAndBenefits"] = expenses_and_benefits
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if xml is not UNSET:
            field_dict["xml"] = xml
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if employer_references is not UNSET:
            field_dict["employerReferences"] = employer_references
        if gov_talk_submission is not UNSET:
            field_dict["govTalkSubmission"] = gov_talk_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        additions_amount = d.pop("additionsAmount", UNSET)

        additions_description = d.pop("additionsDescription", UNSET)

        deductions_amount = d.pop("deductionsAmount", UNSET)

        deductions_description = d.pop("deductionsDescription", UNSET)

        employee_count = d.pop("employeeCount", UNSET)

        total_benefits = d.pop("totalBenefits", UNSET)

        total_adjusted_benefits = d.pop("totalAdjustedBenefits", UNSET)

        nics_rate = d.pop("nicsRate", UNSET)

        nics_payable = d.pop("nicsPayable", UNSET)

        _expenses_and_benefits = d.pop("expensesAndBenefits", UNSET)
        expenses_and_benefits: Union[Unset, ExpensesAndBenefits]
        if isinstance(_expenses_and_benefits, Unset):
            expenses_and_benefits = UNSET
        else:
            expenses_and_benefits = ExpensesAndBenefits.from_dict(
                _expenses_and_benefits
            )

        i_rmark = d.pop("iRmark", UNSET)

        xml = d.pop("xml", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _employer_references = d.pop("employerReferences", UNSET)
        employer_references: Union[Unset, EmpRefs]
        if isinstance(_employer_references, Unset):
            employer_references = UNSET
        else:
            employer_references = EmpRefs.from_dict(_employer_references)

        _gov_talk_submission = d.pop("govTalkSubmission", UNSET)
        gov_talk_submission: Union[Unset, GovTalkSubmission]
        if isinstance(_gov_talk_submission, Unset):
            gov_talk_submission = UNSET
        else:
            gov_talk_submission = GovTalkSubmission.from_dict(_gov_talk_submission)

        id = d.pop("id", UNSET)

        exb = cls(
            additions_amount=additions_amount,
            additions_description=additions_description,
            deductions_amount=deductions_amount,
            deductions_description=deductions_description,
            employee_count=employee_count,
            total_benefits=total_benefits,
            total_adjusted_benefits=total_adjusted_benefits,
            nics_rate=nics_rate,
            nics_payable=nics_payable,
            expenses_and_benefits=expenses_and_benefits,
            i_rmark=i_rmark,
            xml=xml,
            tax_year=tax_year,
            employer_references=employer_references,
            gov_talk_submission=gov_talk_submission,
            id=id,
        )

        return exb



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmpRefs")


@attr.s(auto_attribs=True)
class EmpRefs:
    """
    Attributes:
        office_no (Union[Unset, None, str]):
        paye_ref (Union[Unset, None, str]):
        ao_ref (Union[Unset, None, str]):
        econ (Union[Unset, None, str]):
        cotax_ref (Union[Unset, None, str]):
        sautr (Union[Unset, None, str]):
    """

    office_no: Union[Unset, None, str] = UNSET
    paye_ref: Union[Unset, None, str] = UNSET
    ao_ref: Union[Unset, None, str] = UNSET
    econ: Union[Unset, None, str] = UNSET
    cotax_ref: Union[Unset, None, str] = UNSET
    sautr: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        office_no = self.office_no
        paye_ref = self.paye_ref
        ao_ref = self.ao_ref
        econ = self.econ
        cotax_ref = self.cotax_ref
        sautr = self.sautr

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if office_no is not UNSET:
            field_dict["officeNo"] = office_no
        if paye_ref is not UNSET:
            field_dict["payeRef"] = paye_ref
        if ao_ref is not UNSET:
            field_dict["aoRef"] = ao_ref
        if econ is not UNSET:
            field_dict["econ"] = econ
        if cotax_ref is not UNSET:
            field_dict["cotaxRef"] = cotax_ref
        if sautr is not UNSET:
            field_dict["sautr"] = sautr

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        office_no = d.pop("officeNo", UNSET)

        paye_ref = d.pop("payeRef", UNSET)

        ao_ref = d.pop("aoRef", UNSET)

        econ = d.pop("econ", UNSET)

        cotax_ref = d.pop("cotaxRef", UNSET)

        sautr = d.pop("sautr", UNSET)

        emp_refs = cls(
            office_no=office_no,
            paye_ref=paye_ref,
            ao_ref=ao_ref,
            econ=econ,
            cotax_ref=cotax_ref,
            sautr=sautr,
        )

        return emp_refs



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="CostOfEmploymentReportLine")


@attr.s(auto_attribs=True)
class CostOfEmploymentReportLine:
    """
    Attributes:
        pay (Union[Unset, float]):
        employer_nic (Union[Unset, float]):
        pension (Union[Unset, float]):
        aeo_fees (Union[Unset, float]):
        stat_pay_reclaim (Union[Unset, float]):
        total_cost (Union[Unset, float]):
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
    """

    pay: Union[Unset, float] = UNSET
    employer_nic: Union[Unset, float] = UNSET
    pension: Union[Unset, float] = UNSET
    aeo_fees: Union[Unset, float] = UNSET
    stat_pay_reclaim: Union[Unset, float] = UNSET
    total_cost: Union[Unset, float] = UNSET
    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay = self.pay
        employer_nic = self.employer_nic
        pension = self.pension
        aeo_fees = self.aeo_fees
        stat_pay_reclaim = self.stat_pay_reclaim
        total_cost = self.total_cost
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        department = self.department

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay is not UNSET:
            field_dict["pay"] = pay
        if employer_nic is not UNSET:
            field_dict["employerNic"] = employer_nic
        if pension is not UNSET:
            field_dict["pension"] = pension
        if aeo_fees is not UNSET:
            field_dict["aeoFees"] = aeo_fees
        if stat_pay_reclaim is not UNSET:
            field_dict["statPayReclaim"] = stat_pay_reclaim
        if total_cost is not UNSET:
            field_dict["totalCost"] = total_cost
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pay = d.pop("pay", UNSET)

        employer_nic = d.pop("employerNic", UNSET)

        pension = d.pop("pension", UNSET)

        aeo_fees = d.pop("aeoFees", UNSET)

        stat_pay_reclaim = d.pop("statPayReclaim", UNSET)

        total_cost = d.pop("totalCost", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        cost_of_employment_report_line = cls(
            pay=pay,
            employer_nic=employer_nic,
            pension=pension,
            aeo_fees=aeo_fees,
            stat_pay_reclaim=stat_pay_reclaim,
            total_cost=total_cost,
            employee=employee,
            payroll_code=payroll_code,
            department=department,
        )

        return cost_of_employment_report_line



from enum import Enum


class DpsDataType(str, Enum):
    P6 = "P6"
    P9 = "P9"
    SL1 = "SL1"
    SL2 = "SL2"
    PGL1 = "PGL1"
    PGL2 = "PGL2"
    AR = "AR"
    NOT = "NOT"
    RTI = "RTI"
    CIS = "CIS"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.hmrc_payment import HmrcPayment
from ..models.item import Item
from ..models.pay_run import PayRun
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="HmrcLiability")


@attr.s(auto_attribs=True)
class HmrcLiability:
    """
    Attributes:
        title (Union[Unset, None, str]): [readonly] A descriptive name for the Period
        tax_year (Union[Unset, TaxYear]):
        period_ending (Union[Unset, datetime.date]): [readonly]
        period_complete (Union[Unset, bool]): [readonly] Indicates whether or not all PayRuns for this period have been
            completed
        recoverable_amounts_eps_created (Union[Unset, bool]): [readonly] If there are recoverable amount present (ie,
            SMP) and the Period is complete then this field Indicates
            whether or not an EPS has been created up to the final tax month of this period to declare
            the recoverable amounts
        apprenticeship_levy_eps_created (Union[Unset, bool]): [readonly] If there is an Apprenticeship LEvy amount
            present and the Period is complete then this field Indicates
            whether or not an EPS has been created up to the final tax month of this period to declare
            the amount
        pay_runs (Union[Unset, None, List[PayRun]]):
        tax (Union[Unset, float]): [readonly] Amount due to HMRC for PAYE in period
        cis_deductions (Union[Unset, float]): [readonly] Amount due to HMRC for CIS Deductions made in period
        postgrad_loan_deductions (Union[Unset, float]): [readonly] Amount due to HMRC for Postgrad Loan Deductions made
            in period
        student_loan_deductions (Union[Unset, float]): [readonly] Amount due to HMRC for Student Loan Deductions made in
            period
        employee_nic (Union[Unset, float]): [readonly] Amount due to HMRC for Employee NIC Contributions withheld in
            period
        employer_nic (Union[Unset, float]): [readonly] Amount due to HMRC for Employer NIC Contributions due on payments
            in period
        real_time_class_1a_nic (Union[Unset, float]): [readonly] Amount due to HMRC for Employer NIC Contributions due
            on payments in period
        smp (Union[Unset, float]): [readonly] SMP recovered from payments in period
        smp_nic (Union[Unset, float]): [readonly] SMP NIC Compensation for period
        spp (Union[Unset, float]): [readonly] SPP recovered from payments in period
        spp_nic (Union[Unset, float]): [readonly] SPP Nic Compensation for period
        sap (Union[Unset, float]): [readonly] SAP recovered from payments in period
        sap_nic (Union[Unset, float]): [readonly] SAP Nic Compensation for period
        shpp (Union[Unset, float]): [readonly] ShPP recovered from payments in period
        shpp_nic (Union[Unset, float]): [readonly] ShPP Nic Compensation for period
        spbp (Union[Unset, float]): [readonly] SPBP recovered from payments in period
        spbp_nic (Union[Unset, float]): [readonly] SPBP Nic Compensation for period
        eligible_for_employment_allowance (Union[Unset, bool]): [readonly] Indicates whether the Employer settings say
            the Employer is eligible for Employment Allowance
        employment_allowance_claim (Union[Unset, float]): Amount to claim with respect to the Employment Allowance
        employment_allowance_claim_auto (Union[Unset, bool]): If set to true, we'll automatically claim the maximum
            allowed.
            If set to false then you can enter your own value.
            If EligibleForEmploymentAllowance is false then the EmploymentAllowanceClaim will
            always be overwritten with a zero value regardless of this property
        liable_for_apprenticeship_levy (Union[Unset, bool]): [readonly] Indicates whether the Employer settings say the
            Employer is liable for the Apprenticeship Levy
        apprenticeship_levy (Union[Unset, float]): Amount due with respect to the Apprenticeship Levy
        apprenticeship_levy_auto (Union[Unset, bool]): If set to true, we'll automatically calculate your liability
            If set to false then you can enter your own value.
            If LiableForApprenticeshipLevy is false then the ApprenticeshipLevy will
            always be overwritten with a zero value regardless of this property
        liability_arising_in_period (Union[Unset, float]): [readonly] Total Liability (before any deductions) arising in
            this Period
        due_in_previous_periods (Union[Unset, float]): [readonly] Amount due to HMRC in previous periods for this Tax
            Year
        paid_in_previous_periods (Union[Unset, float]): [readonly] Amount paid to HMRC in previous periods for this Tax
            Year
        employment_allowance_claim_in_previous_periods (Union[Unset, float]): [readonly] The amount of Employment
            Allowance claimed in previous periods for this Tax Year
        received_from_hmrc_to_refund_tax (Union[Unset, float]): Any amount received from HMRC to refund Tax in Period
        received_from_hmrc_to_pay_statutory_pay (Union[Unset, float]): Any amount received from HMRC to pay Statutory
            Pay Period
        cis_deductions_suffered (Union[Unset, float]): CIS Deductions Suffered in Period
        adjustment (Union[Unset, float]): Any manual adjustment for Period
        net_liability_for_period (Union[Unset, float]): [readonly] Net Liability (including any deductions/additions)
            arising in this Period
        total_paid (Union[Unset, float]): [readonly] The total value of all payments in this period
        payments (Union[Unset, None, List[HmrcPayment]]):
        fps_list (Union[Unset, None, List[Item]]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    title: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    period_ending: Union[Unset, datetime.date] = UNSET
    period_complete: Union[Unset, bool] = UNSET
    recoverable_amounts_eps_created: Union[Unset, bool] = UNSET
    apprenticeship_levy_eps_created: Union[Unset, bool] = UNSET
    pay_runs: Union[Unset, None, List[PayRun]] = UNSET
    tax: Union[Unset, float] = UNSET
    cis_deductions: Union[Unset, float] = UNSET
    postgrad_loan_deductions: Union[Unset, float] = UNSET
    student_loan_deductions: Union[Unset, float] = UNSET
    employee_nic: Union[Unset, float] = UNSET
    employer_nic: Union[Unset, float] = UNSET
    real_time_class_1a_nic: Union[Unset, float] = UNSET
    smp: Union[Unset, float] = UNSET
    smp_nic: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    spp_nic: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    sap_nic: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    shpp_nic: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET
    spbp_nic: Union[Unset, float] = UNSET
    eligible_for_employment_allowance: Union[Unset, bool] = UNSET
    employment_allowance_claim: Union[Unset, float] = UNSET
    employment_allowance_claim_auto: Union[Unset, bool] = UNSET
    liable_for_apprenticeship_levy: Union[Unset, bool] = UNSET
    apprenticeship_levy: Union[Unset, float] = UNSET
    apprenticeship_levy_auto: Union[Unset, bool] = UNSET
    liability_arising_in_period: Union[Unset, float] = UNSET
    due_in_previous_periods: Union[Unset, float] = UNSET
    paid_in_previous_periods: Union[Unset, float] = UNSET
    employment_allowance_claim_in_previous_periods: Union[Unset, float] = UNSET
    received_from_hmrc_to_refund_tax: Union[Unset, float] = UNSET
    received_from_hmrc_to_pay_statutory_pay: Union[Unset, float] = UNSET
    cis_deductions_suffered: Union[Unset, float] = UNSET
    adjustment: Union[Unset, float] = UNSET
    net_liability_for_period: Union[Unset, float] = UNSET
    total_paid: Union[Unset, float] = UNSET
    payments: Union[Unset, None, List[HmrcPayment]] = UNSET
    fps_list: Union[Unset, None, List[Item]] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        period_ending: Union[Unset, str] = UNSET
        if not isinstance(self.period_ending, Unset):
            period_ending = self.period_ending.isoformat()

        period_complete = self.period_complete
        recoverable_amounts_eps_created = self.recoverable_amounts_eps_created
        apprenticeship_levy_eps_created = self.apprenticeship_levy_eps_created
        pay_runs: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.pay_runs, Unset):
            if self.pay_runs is None:
                pay_runs = None
            else:
                pay_runs = []
                for pay_runs_item_data in self.pay_runs:
                    pay_runs_item = pay_runs_item_data.to_dict()

                    pay_runs.append(pay_runs_item)

        tax = self.tax
        cis_deductions = self.cis_deductions
        postgrad_loan_deductions = self.postgrad_loan_deductions
        student_loan_deductions = self.student_loan_deductions
        employee_nic = self.employee_nic
        employer_nic = self.employer_nic
        real_time_class_1a_nic = self.real_time_class_1a_nic
        smp = self.smp
        smp_nic = self.smp_nic
        spp = self.spp
        spp_nic = self.spp_nic
        sap = self.sap
        sap_nic = self.sap_nic
        shpp = self.shpp
        shpp_nic = self.shpp_nic
        spbp = self.spbp
        spbp_nic = self.spbp_nic
        eligible_for_employment_allowance = self.eligible_for_employment_allowance
        employment_allowance_claim = self.employment_allowance_claim
        employment_allowance_claim_auto = self.employment_allowance_claim_auto
        liable_for_apprenticeship_levy = self.liable_for_apprenticeship_levy
        apprenticeship_levy = self.apprenticeship_levy
        apprenticeship_levy_auto = self.apprenticeship_levy_auto
        liability_arising_in_period = self.liability_arising_in_period
        due_in_previous_periods = self.due_in_previous_periods
        paid_in_previous_periods = self.paid_in_previous_periods
        employment_allowance_claim_in_previous_periods = (
            self.employment_allowance_claim_in_previous_periods
        )
        received_from_hmrc_to_refund_tax = self.received_from_hmrc_to_refund_tax
        received_from_hmrc_to_pay_statutory_pay = (
            self.received_from_hmrc_to_pay_statutory_pay
        )
        cis_deductions_suffered = self.cis_deductions_suffered
        adjustment = self.adjustment
        net_liability_for_period = self.net_liability_for_period
        total_paid = self.total_paid
        payments: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.payments, Unset):
            if self.payments is None:
                payments = None
            else:
                payments = []
                for payments_item_data in self.payments:
                    payments_item = payments_item_data.to_dict()

                    payments.append(payments_item)

        fps_list: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.fps_list, Unset):
            if self.fps_list is None:
                fps_list = None
            else:
                fps_list = []
                for fps_list_item_data in self.fps_list:
                    fps_list_item = fps_list_item_data.to_dict()

                    fps_list.append(fps_list_item)

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if title is not UNSET:
            field_dict["title"] = title
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if period_ending is not UNSET:
            field_dict["periodEnding"] = period_ending
        if period_complete is not UNSET:
            field_dict["periodComplete"] = period_complete
        if recoverable_amounts_eps_created is not UNSET:
            field_dict["recoverableAmountsEpsCreated"] = recoverable_amounts_eps_created
        if apprenticeship_levy_eps_created is not UNSET:
            field_dict["apprenticeshipLevyEpsCreated"] = apprenticeship_levy_eps_created
        if pay_runs is not UNSET:
            field_dict["payRuns"] = pay_runs
        if tax is not UNSET:
            field_dict["tax"] = tax
        if cis_deductions is not UNSET:
            field_dict["cisDeductions"] = cis_deductions
        if postgrad_loan_deductions is not UNSET:
            field_dict["postgradLoanDeductions"] = postgrad_loan_deductions
        if student_loan_deductions is not UNSET:
            field_dict["studentLoanDeductions"] = student_loan_deductions
        if employee_nic is not UNSET:
            field_dict["employeeNic"] = employee_nic
        if employer_nic is not UNSET:
            field_dict["employerNic"] = employer_nic
        if real_time_class_1a_nic is not UNSET:
            field_dict["realTimeClass1ANic"] = real_time_class_1a_nic
        if smp is not UNSET:
            field_dict["smp"] = smp
        if smp_nic is not UNSET:
            field_dict["smpNic"] = smp_nic
        if spp is not UNSET:
            field_dict["spp"] = spp
        if spp_nic is not UNSET:
            field_dict["sppNic"] = spp_nic
        if sap is not UNSET:
            field_dict["sap"] = sap
        if sap_nic is not UNSET:
            field_dict["sapNic"] = sap_nic
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if shpp_nic is not UNSET:
            field_dict["shppNic"] = shpp_nic
        if spbp is not UNSET:
            field_dict["spbp"] = spbp
        if spbp_nic is not UNSET:
            field_dict["spbpNic"] = spbp_nic
        if eligible_for_employment_allowance is not UNSET:
            field_dict[
                "eligibleForEmploymentAllowance"
            ] = eligible_for_employment_allowance
        if employment_allowance_claim is not UNSET:
            field_dict["employmentAllowanceClaim"] = employment_allowance_claim
        if employment_allowance_claim_auto is not UNSET:
            field_dict["employmentAllowanceClaimAuto"] = employment_allowance_claim_auto
        if liable_for_apprenticeship_levy is not UNSET:
            field_dict["liableForApprenticeshipLevy"] = liable_for_apprenticeship_levy
        if apprenticeship_levy is not UNSET:
            field_dict["apprenticeshipLevy"] = apprenticeship_levy
        if apprenticeship_levy_auto is not UNSET:
            field_dict["apprenticeshipLevyAuto"] = apprenticeship_levy_auto
        if liability_arising_in_period is not UNSET:
            field_dict["liabilityArisingInPeriod"] = liability_arising_in_period
        if due_in_previous_periods is not UNSET:
            field_dict["dueInPreviousPeriods"] = due_in_previous_periods
        if paid_in_previous_periods is not UNSET:
            field_dict["paidInPreviousPeriods"] = paid_in_previous_periods
        if employment_allowance_claim_in_previous_periods is not UNSET:
            field_dict[
                "employmentAllowanceClaimInPreviousPeriods"
            ] = employment_allowance_claim_in_previous_periods
        if received_from_hmrc_to_refund_tax is not UNSET:
            field_dict["receivedFromHMRCToRefundTax"] = received_from_hmrc_to_refund_tax
        if received_from_hmrc_to_pay_statutory_pay is not UNSET:
            field_dict[
                "receivedFromHMRCToPayStatutoryPay"
            ] = received_from_hmrc_to_pay_statutory_pay
        if cis_deductions_suffered is not UNSET:
            field_dict["cisDeductionsSuffered"] = cis_deductions_suffered
        if adjustment is not UNSET:
            field_dict["adjustment"] = adjustment
        if net_liability_for_period is not UNSET:
            field_dict["netLiabilityForPeriod"] = net_liability_for_period
        if total_paid is not UNSET:
            field_dict["totalPaid"] = total_paid
        if payments is not UNSET:
            field_dict["payments"] = payments
        if fps_list is not UNSET:
            field_dict["fpsList"] = fps_list
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _period_ending = d.pop("periodEnding", UNSET)
        period_ending: Union[Unset, datetime.date]
        if isinstance(_period_ending, Unset):
            period_ending = UNSET
        else:
            period_ending = isoparse(_period_ending).date()

        period_complete = d.pop("periodComplete", UNSET)

        recoverable_amounts_eps_created = d.pop("recoverableAmountsEpsCreated", UNSET)

        apprenticeship_levy_eps_created = d.pop("apprenticeshipLevyEpsCreated", UNSET)

        pay_runs = []
        _pay_runs = d.pop("payRuns", UNSET)
        for pay_runs_item_data in _pay_runs or []:
            pay_runs_item = PayRun.from_dict(pay_runs_item_data)

            pay_runs.append(pay_runs_item)

        tax = d.pop("tax", UNSET)

        cis_deductions = d.pop("cisDeductions", UNSET)

        postgrad_loan_deductions = d.pop("postgradLoanDeductions", UNSET)

        student_loan_deductions = d.pop("studentLoanDeductions", UNSET)

        employee_nic = d.pop("employeeNic", UNSET)

        employer_nic = d.pop("employerNic", UNSET)

        real_time_class_1a_nic = d.pop("realTimeClass1ANic", UNSET)

        smp = d.pop("smp", UNSET)

        smp_nic = d.pop("smpNic", UNSET)

        spp = d.pop("spp", UNSET)

        spp_nic = d.pop("sppNic", UNSET)

        sap = d.pop("sap", UNSET)

        sap_nic = d.pop("sapNic", UNSET)

        shpp = d.pop("shpp", UNSET)

        shpp_nic = d.pop("shppNic", UNSET)

        spbp = d.pop("spbp", UNSET)

        spbp_nic = d.pop("spbpNic", UNSET)

        eligible_for_employment_allowance = d.pop(
            "eligibleForEmploymentAllowance", UNSET
        )

        employment_allowance_claim = d.pop("employmentAllowanceClaim", UNSET)

        employment_allowance_claim_auto = d.pop("employmentAllowanceClaimAuto", UNSET)

        liable_for_apprenticeship_levy = d.pop("liableForApprenticeshipLevy", UNSET)

        apprenticeship_levy = d.pop("apprenticeshipLevy", UNSET)

        apprenticeship_levy_auto = d.pop("apprenticeshipLevyAuto", UNSET)

        liability_arising_in_period = d.pop("liabilityArisingInPeriod", UNSET)

        due_in_previous_periods = d.pop("dueInPreviousPeriods", UNSET)

        paid_in_previous_periods = d.pop("paidInPreviousPeriods", UNSET)

        employment_allowance_claim_in_previous_periods = d.pop(
            "employmentAllowanceClaimInPreviousPeriods", UNSET
        )

        received_from_hmrc_to_refund_tax = d.pop("receivedFromHMRCToRefundTax", UNSET)

        received_from_hmrc_to_pay_statutory_pay = d.pop(
            "receivedFromHMRCToPayStatutoryPay", UNSET
        )

        cis_deductions_suffered = d.pop("cisDeductionsSuffered", UNSET)

        adjustment = d.pop("adjustment", UNSET)

        net_liability_for_period = d.pop("netLiabilityForPeriod", UNSET)

        total_paid = d.pop("totalPaid", UNSET)

        payments = []
        _payments = d.pop("payments", UNSET)
        for payments_item_data in _payments or []:
            payments_item = HmrcPayment.from_dict(payments_item_data)

            payments.append(payments_item)

        fps_list = []
        _fps_list = d.pop("fpsList", UNSET)
        for fps_list_item_data in _fps_list or []:
            fps_list_item = Item.from_dict(fps_list_item_data)

            fps_list.append(fps_list_item)

        id = d.pop("id", UNSET)

        hmrc_liability = cls(
            title=title,
            tax_year=tax_year,
            period_ending=period_ending,
            period_complete=period_complete,
            recoverable_amounts_eps_created=recoverable_amounts_eps_created,
            apprenticeship_levy_eps_created=apprenticeship_levy_eps_created,
            pay_runs=pay_runs,
            tax=tax,
            cis_deductions=cis_deductions,
            postgrad_loan_deductions=postgrad_loan_deductions,
            student_loan_deductions=student_loan_deductions,
            employee_nic=employee_nic,
            employer_nic=employer_nic,
            real_time_class_1a_nic=real_time_class_1a_nic,
            smp=smp,
            smp_nic=smp_nic,
            spp=spp,
            spp_nic=spp_nic,
            sap=sap,
            sap_nic=sap_nic,
            shpp=shpp,
            shpp_nic=shpp_nic,
            spbp=spbp,
            spbp_nic=spbp_nic,
            eligible_for_employment_allowance=eligible_for_employment_allowance,
            employment_allowance_claim=employment_allowance_claim,
            employment_allowance_claim_auto=employment_allowance_claim_auto,
            liable_for_apprenticeship_levy=liable_for_apprenticeship_levy,
            apprenticeship_levy=apprenticeship_levy,
            apprenticeship_levy_auto=apprenticeship_levy_auto,
            liability_arising_in_period=liability_arising_in_period,
            due_in_previous_periods=due_in_previous_periods,
            paid_in_previous_periods=paid_in_previous_periods,
            employment_allowance_claim_in_previous_periods=employment_allowance_claim_in_previous_periods,
            received_from_hmrc_to_refund_tax=received_from_hmrc_to_refund_tax,
            received_from_hmrc_to_pay_statutory_pay=received_from_hmrc_to_pay_statutory_pay,
            cis_deductions_suffered=cis_deductions_suffered,
            adjustment=adjustment,
            net_liability_for_period=net_liability_for_period,
            total_paid=total_paid,
            payments=payments,
            fps_list=fps_list,
            id=id,
        )

        return hmrc_liability



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.external_data_provider_id import ExternalDataProviderId
from ..models.item import Item
from ..models.tax_year import TaxYear
from ..models.year_end_tax_code_change import YearEndTaxCodeChange
from ..types import UNSET, Unset

T = TypeVar("T", bound="YearEnd")


@attr.s(auto_attribs=True)
class YearEnd:
    """This model gives you a summary of what will happen when you confirm the closing of one year and the start of the
    next

        Attributes:
            ending_year (Union[Unset, TaxYear]):
            starting_year (Union[Unset, TaxYear]):
            create_eps_for_final_submission (Union[Unset, bool]): [readonly] Whether or not the system will automatically
                create an EPS to tell HMRC the year has ended.
            create_eps_for_employment_allowance (Union[Unset, bool]): [readonly] Whether or not the system will
                automatically create an EPS to tell HMRC you qualify for Employment Allowance.
            create_exb (Union[Unset, bool]): [readonly] Whether or not the system will automatically create an EXB to inform
                HMRC of Expenses and Benefits
            set_employment_allowance (Union[Unset, None, float]): [readonly] If the Employment Allowance needs to be
                changed, this indicates the new value
            tax_code_changes (Union[Unset, None, List[YearEndTaxCodeChange]]): [readonly] Details of changes that wil be
                made to Tax Codes
            remove_week_1_month_1 (Union[Unset, None, List[Item]]): [readonly] Employees that will have the Week1Month1 flag
                removed from their tax code
            email_p60 (Union[Unset, None, List[Item]]): [readonly] Employees who will be automatically emailed P60s
            push_p60 (Union[Unset, None, List[ExternalDataProviderId]]): [readonly] ExternalDataProviderIds to which P60s
                can be pushed
            email_cis_statement (Union[Unset, None, List[Item]]): [readonly] Subcontractors who will be automatically sent
                an annual CIS Statement
    """

    ending_year: Union[Unset, TaxYear] = UNSET
    starting_year: Union[Unset, TaxYear] = UNSET
    create_eps_for_final_submission: Union[Unset, bool] = UNSET
    create_eps_for_employment_allowance: Union[Unset, bool] = UNSET
    create_exb: Union[Unset, bool] = UNSET
    set_employment_allowance: Union[Unset, None, float] = UNSET
    tax_code_changes: Union[Unset, None, List[YearEndTaxCodeChange]] = UNSET
    remove_week_1_month_1: Union[Unset, None, List[Item]] = UNSET
    email_p60: Union[Unset, None, List[Item]] = UNSET
    push_p60: Union[Unset, None, List[ExternalDataProviderId]] = UNSET
    email_cis_statement: Union[Unset, None, List[Item]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        ending_year: Union[Unset, str] = UNSET
        if not isinstance(self.ending_year, Unset):
            ending_year = self.ending_year.value

        starting_year: Union[Unset, str] = UNSET
        if not isinstance(self.starting_year, Unset):
            starting_year = self.starting_year.value

        create_eps_for_final_submission = self.create_eps_for_final_submission
        create_eps_for_employment_allowance = self.create_eps_for_employment_allowance
        create_exb = self.create_exb
        set_employment_allowance = self.set_employment_allowance
        tax_code_changes: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.tax_code_changes, Unset):
            if self.tax_code_changes is None:
                tax_code_changes = None
            else:
                tax_code_changes = []
                for tax_code_changes_item_data in self.tax_code_changes:
                    tax_code_changes_item = tax_code_changes_item_data.to_dict()

                    tax_code_changes.append(tax_code_changes_item)

        remove_week_1_month_1: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.remove_week_1_month_1, Unset):
            if self.remove_week_1_month_1 is None:
                remove_week_1_month_1 = None
            else:
                remove_week_1_month_1 = []
                for remove_week_1_month_1_item_data in self.remove_week_1_month_1:
                    remove_week_1_month_1_item = (
                        remove_week_1_month_1_item_data.to_dict()
                    )

                    remove_week_1_month_1.append(remove_week_1_month_1_item)

        email_p60: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.email_p60, Unset):
            if self.email_p60 is None:
                email_p60 = None
            else:
                email_p60 = []
                for email_p60_item_data in self.email_p60:
                    email_p60_item = email_p60_item_data.to_dict()

                    email_p60.append(email_p60_item)

        push_p60: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.push_p60, Unset):
            if self.push_p60 is None:
                push_p60 = None
            else:
                push_p60 = []
                for push_p60_item_data in self.push_p60:
                    push_p60_item = push_p60_item_data.value

                    push_p60.append(push_p60_item)

        email_cis_statement: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.email_cis_statement, Unset):
            if self.email_cis_statement is None:
                email_cis_statement = None
            else:
                email_cis_statement = []
                for email_cis_statement_item_data in self.email_cis_statement:
                    email_cis_statement_item = email_cis_statement_item_data.to_dict()

                    email_cis_statement.append(email_cis_statement_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if ending_year is not UNSET:
            field_dict["endingYear"] = ending_year
        if starting_year is not UNSET:
            field_dict["startingYear"] = starting_year
        if create_eps_for_final_submission is not UNSET:
            field_dict["createEpsForFinalSubmission"] = create_eps_for_final_submission
        if create_eps_for_employment_allowance is not UNSET:
            field_dict[
                "createEpsForEmploymentAllowance"
            ] = create_eps_for_employment_allowance
        if create_exb is not UNSET:
            field_dict["createExb"] = create_exb
        if set_employment_allowance is not UNSET:
            field_dict["setEmploymentAllowance"] = set_employment_allowance
        if tax_code_changes is not UNSET:
            field_dict["taxCodeChanges"] = tax_code_changes
        if remove_week_1_month_1 is not UNSET:
            field_dict["removeWeek1Month1"] = remove_week_1_month_1
        if email_p60 is not UNSET:
            field_dict["emailP60"] = email_p60
        if push_p60 is not UNSET:
            field_dict["pushP60"] = push_p60
        if email_cis_statement is not UNSET:
            field_dict["emailCisStatement"] = email_cis_statement

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _ending_year = d.pop("endingYear", UNSET)
        ending_year: Union[Unset, TaxYear]
        if isinstance(_ending_year, Unset):
            ending_year = UNSET
        else:
            ending_year = TaxYear(_ending_year)

        _starting_year = d.pop("startingYear", UNSET)
        starting_year: Union[Unset, TaxYear]
        if isinstance(_starting_year, Unset):
            starting_year = UNSET
        else:
            starting_year = TaxYear(_starting_year)

        create_eps_for_final_submission = d.pop("createEpsForFinalSubmission", UNSET)

        create_eps_for_employment_allowance = d.pop(
            "createEpsForEmploymentAllowance", UNSET
        )

        create_exb = d.pop("createExb", UNSET)

        set_employment_allowance = d.pop("setEmploymentAllowance", UNSET)

        tax_code_changes = []
        _tax_code_changes = d.pop("taxCodeChanges", UNSET)
        for tax_code_changes_item_data in _tax_code_changes or []:
            tax_code_changes_item = YearEndTaxCodeChange.from_dict(
                tax_code_changes_item_data
            )

            tax_code_changes.append(tax_code_changes_item)

        remove_week_1_month_1 = []
        _remove_week_1_month_1 = d.pop("removeWeek1Month1", UNSET)
        for remove_week_1_month_1_item_data in _remove_week_1_month_1 or []:
            remove_week_1_month_1_item = Item.from_dict(remove_week_1_month_1_item_data)

            remove_week_1_month_1.append(remove_week_1_month_1_item)

        email_p60 = []
        _email_p60 = d.pop("emailP60", UNSET)
        for email_p60_item_data in _email_p60 or []:
            email_p60_item = Item.from_dict(email_p60_item_data)

            email_p60.append(email_p60_item)

        push_p60 = []
        _push_p60 = d.pop("pushP60", UNSET)
        for push_p60_item_data in _push_p60 or []:
            push_p60_item = ExternalDataProviderId(push_p60_item_data)

            push_p60.append(push_p60_item)

        email_cis_statement = []
        _email_cis_statement = d.pop("emailCisStatement", UNSET)
        for email_cis_statement_item_data in _email_cis_statement or []:
            email_cis_statement_item = Item.from_dict(email_cis_statement_item_data)

            email_cis_statement.append(email_cis_statement_item)

        year_end = cls(
            ending_year=ending_year,
            starting_year=starting_year,
            create_eps_for_final_submission=create_eps_for_final_submission,
            create_eps_for_employment_allowance=create_eps_for_employment_allowance,
            create_exb=create_exb,
            set_employment_allowance=set_employment_allowance,
            tax_code_changes=tax_code_changes,
            remove_week_1_month_1=remove_week_1_month_1,
            email_p60=email_p60,
            push_p60=push_p60,
            email_cis_statement=email_cis_statement,
        )

        return year_end



from enum import Enum


class LeavePayType(str, Enum):
    DONOTPAY = "DoNotPay"
    PAYASUSUAL = "PayAsUsual"
    STATUTORYPAY = "StatutoryPay"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.cis_request import CisRequest
from ..models.cis_sub_contractor import CisSubContractor
from ..models.emp_refs import EmpRefs
from ..models.gov_talk_submission import GovTalkSubmission
from ..models.item import Item
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="CisVerification")


@attr.s(auto_attribs=True)
class CisVerification:
    """If an Employee is marked as a CIS Subcontractor then this model is included ad a child of CisDetails and is used to
    represent the Verification details of the Subcontractor

        Attributes:
            employees (Union[Unset, None, List[Item]]):
            employee_count (Union[Unset, int]):
            cis_request (Union[Unset, CisRequest]):
            responses (Union[Unset, None, List[CisSubContractor]]):
            i_rmark (Union[Unset, None, str]):
            xml (Union[Unset, None, str]):
            tax_year (Union[Unset, TaxYear]):
            employer_references (Union[Unset, EmpRefs]):
            gov_talk_submission (Union[Unset, GovTalkSubmission]):
            id (Union[Unset, str]): [readonly] The unique id of the object
    """

    employees: Union[Unset, None, List[Item]] = UNSET
    employee_count: Union[Unset, int] = UNSET
    cis_request: Union[Unset, CisRequest] = UNSET
    responses: Union[Unset, None, List[CisSubContractor]] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    xml: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    employer_references: Union[Unset, EmpRefs] = UNSET
    gov_talk_submission: Union[Unset, GovTalkSubmission] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employees: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.employees, Unset):
            if self.employees is None:
                employees = None
            else:
                employees = []
                for employees_item_data in self.employees:
                    employees_item = employees_item_data.to_dict()

                    employees.append(employees_item)

        employee_count = self.employee_count
        cis_request: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cis_request, Unset):
            cis_request = self.cis_request.to_dict()

        responses: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responses, Unset):
            if self.responses is None:
                responses = None
            else:
                responses = []
                for responses_item_data in self.responses:
                    responses_item = responses_item_data.to_dict()

                    responses.append(responses_item)

        i_rmark = self.i_rmark
        xml = self.xml
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        employer_references: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_references, Unset):
            employer_references = self.employer_references.to_dict()

        gov_talk_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gov_talk_submission, Unset):
            gov_talk_submission = self.gov_talk_submission.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employees is not UNSET:
            field_dict["employees"] = employees
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if cis_request is not UNSET:
            field_dict["cisRequest"] = cis_request
        if responses is not UNSET:
            field_dict["responses"] = responses
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if xml is not UNSET:
            field_dict["xml"] = xml
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if employer_references is not UNSET:
            field_dict["employerReferences"] = employer_references
        if gov_talk_submission is not UNSET:
            field_dict["govTalkSubmission"] = gov_talk_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employees = []
        _employees = d.pop("employees", UNSET)
        for employees_item_data in _employees or []:
            employees_item = Item.from_dict(employees_item_data)

            employees.append(employees_item)

        employee_count = d.pop("employeeCount", UNSET)

        _cis_request = d.pop("cisRequest", UNSET)
        cis_request: Union[Unset, CisRequest]
        if isinstance(_cis_request, Unset):
            cis_request = UNSET
        else:
            cis_request = CisRequest.from_dict(_cis_request)

        responses = []
        _responses = d.pop("responses", UNSET)
        for responses_item_data in _responses or []:
            responses_item = CisSubContractor.from_dict(responses_item_data)

            responses.append(responses_item)

        i_rmark = d.pop("iRmark", UNSET)

        xml = d.pop("xml", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _employer_references = d.pop("employerReferences", UNSET)
        employer_references: Union[Unset, EmpRefs]
        if isinstance(_employer_references, Unset):
            employer_references = UNSET
        else:
            employer_references = EmpRefs.from_dict(_employer_references)

        _gov_talk_submission = d.pop("govTalkSubmission", UNSET)
        gov_talk_submission: Union[Unset, GovTalkSubmission]
        if isinstance(_gov_talk_submission, Unset):
            gov_talk_submission = UNSET
        else:
            gov_talk_submission = GovTalkSubmission.from_dict(_gov_talk_submission)

        id = d.pop("id", UNSET)

        cis_verification = cls(
            employees=employees,
            employee_count=employee_count,
            cis_request=cis_request,
            responses=responses,
            i_rmark=i_rmark,
            xml=xml,
            tax_year=tax_year,
            employer_references=employer_references,
            gov_talk_submission=gov_talk_submission,
            id=id,
        )

        return cis_verification



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.cookie_preference import CookiePreference
from ..models.userstart_page import UserstartPage
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserDisplayPreferences")


@attr.s(auto_attribs=True)
class UserDisplayPreferences:
    """
    Attributes:
        start_page (Union[Unset, UserstartPage]):
        show_zeroes (Union[Unset, bool]):
        show_tax_code_when_viewing_payrun_entry (Union[Unset, bool]):
        allow_journal_resubmit (Union[Unset, bool]):
        hide_salary_on_employee_index_page (Union[Unset, bool]):
        enable_multi_employer_import (Union[Unset, bool]):
        enable_covid_19_features (Union[Unset, bool]):
        hide_payslip_checkboxes (Union[Unset, bool]):
        hide_payslip_search_and_sort (Union[Unset, bool]):
        show_ytd_in_payslip_view (Union[Unset, bool]):
        enable_dps_xml_upload (Union[Unset, bool]):
        enable_rti_timestamp_override (Union[Unset, bool]):
        enable_payrun_warnings (Union[Unset, bool]):
        enable_working_days_overrides (Union[Unset, bool]):
        cookie_preference (Union[Unset, CookiePreference]):
    """

    start_page: Union[Unset, UserstartPage] = UNSET
    show_zeroes: Union[Unset, bool] = UNSET
    show_tax_code_when_viewing_payrun_entry: Union[Unset, bool] = UNSET
    allow_journal_resubmit: Union[Unset, bool] = UNSET
    hide_salary_on_employee_index_page: Union[Unset, bool] = UNSET
    enable_multi_employer_import: Union[Unset, bool] = UNSET
    enable_covid_19_features: Union[Unset, bool] = UNSET
    hide_payslip_checkboxes: Union[Unset, bool] = UNSET
    hide_payslip_search_and_sort: Union[Unset, bool] = UNSET
    show_ytd_in_payslip_view: Union[Unset, bool] = UNSET
    enable_dps_xml_upload: Union[Unset, bool] = UNSET
    enable_rti_timestamp_override: Union[Unset, bool] = UNSET
    enable_payrun_warnings: Union[Unset, bool] = UNSET
    enable_working_days_overrides: Union[Unset, bool] = UNSET
    cookie_preference: Union[Unset, CookiePreference] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start_page: Union[Unset, str] = UNSET
        if not isinstance(self.start_page, Unset):
            start_page = self.start_page.value

        show_zeroes = self.show_zeroes
        show_tax_code_when_viewing_payrun_entry = (
            self.show_tax_code_when_viewing_payrun_entry
        )
        allow_journal_resubmit = self.allow_journal_resubmit
        hide_salary_on_employee_index_page = self.hide_salary_on_employee_index_page
        enable_multi_employer_import = self.enable_multi_employer_import
        enable_covid_19_features = self.enable_covid_19_features
        hide_payslip_checkboxes = self.hide_payslip_checkboxes
        hide_payslip_search_and_sort = self.hide_payslip_search_and_sort
        show_ytd_in_payslip_view = self.show_ytd_in_payslip_view
        enable_dps_xml_upload = self.enable_dps_xml_upload
        enable_rti_timestamp_override = self.enable_rti_timestamp_override
        enable_payrun_warnings = self.enable_payrun_warnings
        enable_working_days_overrides = self.enable_working_days_overrides
        cookie_preference: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.cookie_preference, Unset):
            cookie_preference = self.cookie_preference.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if start_page is not UNSET:
            field_dict["startPage"] = start_page
        if show_zeroes is not UNSET:
            field_dict["showZeroes"] = show_zeroes
        if show_tax_code_when_viewing_payrun_entry is not UNSET:
            field_dict[
                "showTaxCodeWhenViewingPayrunEntry"
            ] = show_tax_code_when_viewing_payrun_entry
        if allow_journal_resubmit is not UNSET:
            field_dict["allowJournalResubmit"] = allow_journal_resubmit
        if hide_salary_on_employee_index_page is not UNSET:
            field_dict[
                "hideSalaryOnEmployeeIndexPage"
            ] = hide_salary_on_employee_index_page
        if enable_multi_employer_import is not UNSET:
            field_dict["enableMultiEmployerImport"] = enable_multi_employer_import
        if enable_covid_19_features is not UNSET:
            field_dict["enableCovid19Features"] = enable_covid_19_features
        if hide_payslip_checkboxes is not UNSET:
            field_dict["hidePayslipCheckboxes"] = hide_payslip_checkboxes
        if hide_payslip_search_and_sort is not UNSET:
            field_dict["hidePayslipSearchAndSort"] = hide_payslip_search_and_sort
        if show_ytd_in_payslip_view is not UNSET:
            field_dict["showYtdInPayslipView"] = show_ytd_in_payslip_view
        if enable_dps_xml_upload is not UNSET:
            field_dict["enableDpsXmlUpload"] = enable_dps_xml_upload
        if enable_rti_timestamp_override is not UNSET:
            field_dict["enableRtiTimestampOverride"] = enable_rti_timestamp_override
        if enable_payrun_warnings is not UNSET:
            field_dict["enablePayrunWarnings"] = enable_payrun_warnings
        if enable_working_days_overrides is not UNSET:
            field_dict["enableWorkingDaysOverrides"] = enable_working_days_overrides
        if cookie_preference is not UNSET:
            field_dict["cookiePreference"] = cookie_preference

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _start_page = d.pop("startPage", UNSET)
        start_page: Union[Unset, UserstartPage]
        if isinstance(_start_page, Unset):
            start_page = UNSET
        else:
            start_page = UserstartPage(_start_page)

        show_zeroes = d.pop("showZeroes", UNSET)

        show_tax_code_when_viewing_payrun_entry = d.pop(
            "showTaxCodeWhenViewingPayrunEntry", UNSET
        )

        allow_journal_resubmit = d.pop("allowJournalResubmit", UNSET)

        hide_salary_on_employee_index_page = d.pop(
            "hideSalaryOnEmployeeIndexPage", UNSET
        )

        enable_multi_employer_import = d.pop("enableMultiEmployerImport", UNSET)

        enable_covid_19_features = d.pop("enableCovid19Features", UNSET)

        hide_payslip_checkboxes = d.pop("hidePayslipCheckboxes", UNSET)

        hide_payslip_search_and_sort = d.pop("hidePayslipSearchAndSort", UNSET)

        show_ytd_in_payslip_view = d.pop("showYtdInPayslipView", UNSET)

        enable_dps_xml_upload = d.pop("enableDpsXmlUpload", UNSET)

        enable_rti_timestamp_override = d.pop("enableRtiTimestampOverride", UNSET)

        enable_payrun_warnings = d.pop("enablePayrunWarnings", UNSET)

        enable_working_days_overrides = d.pop("enableWorkingDaysOverrides", UNSET)

        _cookie_preference = d.pop("cookiePreference", UNSET)
        cookie_preference: Union[Unset, CookiePreference]
        if isinstance(_cookie_preference, Unset):
            cookie_preference = UNSET
        else:
            cookie_preference = CookiePreference.from_dict(_cookie_preference)

        user_display_preferences = cls(
            start_page=start_page,
            show_zeroes=show_zeroes,
            show_tax_code_when_viewing_payrun_entry=show_tax_code_when_viewing_payrun_entry,
            allow_journal_resubmit=allow_journal_resubmit,
            hide_salary_on_employee_index_page=hide_salary_on_employee_index_page,
            enable_multi_employer_import=enable_multi_employer_import,
            enable_covid_19_features=enable_covid_19_features,
            hide_payslip_checkboxes=hide_payslip_checkboxes,
            hide_payslip_search_and_sort=hide_payslip_search_and_sort,
            show_ytd_in_payslip_view=show_ytd_in_payslip_view,
            enable_dps_xml_upload=enable_dps_xml_upload,
            enable_rti_timestamp_override=enable_rti_timestamp_override,
            enable_payrun_warnings=enable_payrun_warnings,
            enable_working_days_overrides=enable_working_days_overrides,
            cookie_preference=cookie_preference,
        )

        return user_display_preferences



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.pay_schedule_period_event import PaySchedulePeriodEvent
from ..types import UNSET, Unset

T = TypeVar("T", bound="PaySchedulePeriod")


@attr.s(auto_attribs=True)
class PaySchedulePeriod:
    """
    Attributes:
        pay_schedule_period_events (Union[Unset, None, List[PaySchedulePeriodEvent]]): [readonly] List of all the events
            in this PaySchedulePeriod
        period (Union[Unset, int]): [readonly] The Period number of the PaySchedulePeriod.
        start_date (Union[Unset, datetime.date]): [readonly] The start date of the PaySchedulePeriod
        end_date (Union[Unset, datetime.date]): [readonly] The end date of the PaySchedulePeriod
        payment_date (Union[Unset, datetime.date]): The payment date of the PaySchedulePeriod.
        unadjusted_payment_date (Union[Unset, datetime.date]): The payment date of the PaySchedulePeriod when not
            accounting for weekends, bank hols or ad-hoc alterations to the pay schedule
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    pay_schedule_period_events: Union[Unset, None, List[PaySchedulePeriodEvent]] = UNSET
    period: Union[Unset, int] = UNSET
    start_date: Union[Unset, datetime.date] = UNSET
    end_date: Union[Unset, datetime.date] = UNSET
    payment_date: Union[Unset, datetime.date] = UNSET
    unadjusted_payment_date: Union[Unset, datetime.date] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        pay_schedule_period_events: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.pay_schedule_period_events, Unset):
            if self.pay_schedule_period_events is None:
                pay_schedule_period_events = None
            else:
                pay_schedule_period_events = []
                for (
                    pay_schedule_period_events_item_data
                ) in self.pay_schedule_period_events:
                    pay_schedule_period_events_item = (
                        pay_schedule_period_events_item_data.to_dict()
                    )

                    pay_schedule_period_events.append(pay_schedule_period_events_item)

        period = self.period
        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date, Unset):
            payment_date = self.payment_date.isoformat()

        unadjusted_payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.unadjusted_payment_date, Unset):
            unadjusted_payment_date = self.unadjusted_payment_date.isoformat()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if pay_schedule_period_events is not UNSET:
            field_dict["paySchedulePeriodEvents"] = pay_schedule_period_events
        if period is not UNSET:
            field_dict["period"] = period
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if payment_date is not UNSET:
            field_dict["paymentDate"] = payment_date
        if unadjusted_payment_date is not UNSET:
            field_dict["unadjustedPaymentDate"] = unadjusted_payment_date
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        pay_schedule_period_events = []
        _pay_schedule_period_events = d.pop("paySchedulePeriodEvents", UNSET)
        for pay_schedule_period_events_item_data in _pay_schedule_period_events or []:
            pay_schedule_period_events_item = PaySchedulePeriodEvent.from_dict(
                pay_schedule_period_events_item_data
            )

            pay_schedule_period_events.append(pay_schedule_period_events_item)

        period = d.pop("period", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.date]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date).date()

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.date]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date).date()

        _payment_date = d.pop("paymentDate", UNSET)
        payment_date: Union[Unset, datetime.date]
        if isinstance(_payment_date, Unset):
            payment_date = UNSET
        else:
            payment_date = isoparse(_payment_date).date()

        _unadjusted_payment_date = d.pop("unadjustedPaymentDate", UNSET)
        unadjusted_payment_date: Union[Unset, datetime.date]
        if isinstance(_unadjusted_payment_date, Unset):
            unadjusted_payment_date = UNSET
        else:
            unadjusted_payment_date = isoparse(_unadjusted_payment_date).date()

        id = d.pop("id", UNSET)

        pay_schedule_period = cls(
            pay_schedule_period_events=pay_schedule_period_events,
            period=period,
            start_date=start_date,
            end_date=end_date,
            payment_date=payment_date,
            unadjusted_payment_date=unadjusted_payment_date,
            id=id,
        )

        return pay_schedule_period



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.html_insertion_point import HtmlInsertionPoint
from ..types import UNSET, Unset

T = TypeVar("T", bound="TenantHtmlInsertion")


@attr.s(auto_attribs=True)
class TenantHtmlInsertion:
    """
    Attributes:
        insertion_point (Union[Unset, HtmlInsertionPoint]):
        content (Union[Unset, None, str]):
    """

    insertion_point: Union[Unset, HtmlInsertionPoint] = UNSET
    content: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        insertion_point: Union[Unset, str] = UNSET
        if not isinstance(self.insertion_point, Unset):
            insertion_point = self.insertion_point.value

        content = self.content

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if insertion_point is not UNSET:
            field_dict["insertionPoint"] = insertion_point
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _insertion_point = d.pop("insertionPoint", UNSET)
        insertion_point: Union[Unset, HtmlInsertionPoint]
        if isinstance(_insertion_point, Unset):
            insertion_point = UNSET
        else:
            insertion_point = HtmlInsertionPoint(_insertion_point)

        content = d.pop("content", UNSET)

        tenant_html_insertion = cls(
            insertion_point=insertion_point,
            content=content,
        )

        return tenant_html_insertion



from typing import Any, Dict, List, Union

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...models.pay_options_import import PayOptionsImport
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/import/pay".format(
        client.base_url, employerId=employer_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["linesOnly"] = lines_only

    params["append"] = append

    params["updateExisting"] = update_existing

    params["validateOnly"] = validate_only

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = []
    for json_body_item_data in json_body:
        json_body_item = json_body_item_data.to_dict()

        json_json_body.append(json_body_item)

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Import Pay To Employee

     Takes a list PayOptionsImport and updates employees PayOptions to use the values provided.

    Args:
        employer_id (str):
        lines_only (Union[Unset, None, bool]):
        append (Union[Unset, None, bool]):
        update_existing (Union[Unset, None, bool]):
        validate_only (Union[Unset, None, bool]):
        json_body (List[PayOptionsImport]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
        lines_only=lines_only,
        append=append,
        update_existing=update_existing,
        validate_only=validate_only,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    employer_id: str,
    *,
    client: Client,
    json_body: List[PayOptionsImport],
    lines_only: Union[Unset, None, bool] = False,
    append: Union[Unset, None, bool] = False,
    update_existing: Union[Unset, None, bool] = False,
    validate_only: Union[Unset, None, bool] = False,
) -> Response[Any]:
    """Import Pay To Employee

     Takes a list PayOptionsImport and updates employees PayOptions to use the values provided.

    Args:
        employer_id (str):
        lines_only (Union[Unset, None, bool]):
        append (Union[Unset, None, bool]):
        update_existing (Union[Unset, None, bool]):
        validate_only (Union[Unset, None, bool]):
        json_body (List[PayOptionsImport]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        client=client,
        json_body=json_body,
        lines_only=lines_only,
        append=append,
        update_existing=update_existing,
        validate_only=validate_only,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from enum import Enum


class PdfPasswordType(str, Enum):
    INITIALSANDDOB = "InitialsAndDob"
    NINUMBER = "NiNumber"
    CUSTOM = "Custom"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict

import httpx
from staffology.propagate_exceptions import raise_staffology_exception

from ...client import Client
from ...types import Response


def _get_kwargs(
    id: str,
    user_id: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/employers/{id}/users/{userId}".format(
        client.base_url, id=id, userId=user_id
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    id: str,
    user_id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Remove User

     Removes a User from an Employer.
    You cannot remove Users that are marked as the owner of the Employer.
    You must be the owner of the Employer in order to remove other Users.

    Args:
        id (str):
        user_id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        user_id=user_id,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    id: str,
    user_id: str,
    *,
    client: Client,
) -> Response[Any]:
    """Remove User

     Removes a User from an Employer.
    You cannot remove Users that are marked as the owner of the Employer.
    You must be the owner of the Employer in order to remove other Users.

    Args:
        id (str):
        user_id (str):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        user_id=user_id,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from enum import Enum


class NiLetterError(str, Enum):
    XNOTUSEDFORPENSIONERSPAYROLL = "XNotUsedForPensionersPayroll"
    BUSEDFORMALE = "BUsedForMale"
    CUSEDBEFORESTATEPENSIONAGE = "CUsedBeforeStatePensionAge"
    AUSEDFORUNDER21 = "AUsedForUnder21"
    AUSEDOVERSTATEPENSIONAGE = "AUsedOverStatePensionAge"
    HUSEDFOROVER24 = "HUsedForOver24"
    MUSEDFOROVER20 = "MUsedForOver20"
    ZUSEDFOROVER20 = "ZUsedForOver20"
    IUSEDFORMALE = "IUsedForMale"
    FUSEDFORNONFREEPORTWORKER = "FUsedForNonFreeportWorker"
    IUSEDFORNONFREEPORTWORKER = "IUsedForNonFreeportWorker"
    LUSEDFORNONFREEPORTWORKER = "LUsedForNonFreeportWorker"
    SUSEDFORNONFREEPORTWORKER = "SUsedForNonFreeportWorker"
    VUSEDFORNONVETERAN = "VUsedForNonVeteran"
    VUSEDFORVETERANSFIRSTEMPLOYMENTDATEOVERONEYEAROLD = (
        "VUsedForVeteransFirstEmploymentDateOverOneYearOld"
    )
    FUSEDFORFREEPORTWORKEROVERTHREEYEARSOLD = "FUsedForFreeportWorkerOverThreeYearsOld"
    IUSEDFORFREEPORTWORKEROVERTHREEYEARSOLD = "IUsedForFreeportWorkerOverThreeYearsOld"
    LUSEDFORFREEPORTWORKEROVERTHREEYEARSOLD = "LUsedForFreeportWorkerOverThreeYearsOld"
    SUSEDFORFREEPORTWORKEROVERTHREEYEARSOLD = "SUsedForFreeportWorkerOverThreeYearsOld"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.emp_refs import EmpRefs
from ..models.eps_account import EpsAccount
from ..models.eps_apprenticeship_levy import EpsApprenticeshipLevy
from ..models.eps_de_minimis_state_aid import EpsDeMinimisStateAid
from ..models.eps_final_submission import EpsFinalSubmission
from ..models.from_to_dates import FromToDates
from ..models.gov_talk_submission import GovTalkSubmission
from ..models.recoverable_amounts import RecoverableAmounts
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="Eps")


@attr.s(auto_attribs=True)
class Eps:
    """
    Attributes:
        period_of_inactivity (Union[Unset, FromToDates]):
        no_payment_for_period (Union[Unset, FromToDates]):
        final_submission (Union[Unset, EpsFinalSubmission]): Used on an EPS to declare a Final Submission
        recoverable_amounts (Union[Unset, RecoverableAmounts]):
        apprenticeship_levy (Union[Unset, EpsApprenticeshipLevy]): Used on an EPS to declare an Apprenticeship Levy
            amount
        account (Union[Unset, EpsAccount]): Used on an EPS to send bank account information
        eligible_for_employment_allowance (Union[Unset, None, bool]):
        de_minimis_state_aid (Union[Unset, EpsDeMinimisStateAid]): Used on an EPS to declare an Employment Allowance
            DeMinimis State Aid information
        i_rmark (Union[Unset, None, str]):
        xml (Union[Unset, None, str]):
        tax_year (Union[Unset, TaxYear]):
        employer_references (Union[Unset, EmpRefs]):
        gov_talk_submission (Union[Unset, GovTalkSubmission]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    period_of_inactivity: Union[Unset, FromToDates] = UNSET
    no_payment_for_period: Union[Unset, FromToDates] = UNSET
    final_submission: Union[Unset, EpsFinalSubmission] = UNSET
    recoverable_amounts: Union[Unset, RecoverableAmounts] = UNSET
    apprenticeship_levy: Union[Unset, EpsApprenticeshipLevy] = UNSET
    account: Union[Unset, EpsAccount] = UNSET
    eligible_for_employment_allowance: Union[Unset, None, bool] = UNSET
    de_minimis_state_aid: Union[Unset, EpsDeMinimisStateAid] = UNSET
    i_rmark: Union[Unset, None, str] = UNSET
    xml: Union[Unset, None, str] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    employer_references: Union[Unset, EmpRefs] = UNSET
    gov_talk_submission: Union[Unset, GovTalkSubmission] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        period_of_inactivity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.period_of_inactivity, Unset):
            period_of_inactivity = self.period_of_inactivity.to_dict()

        no_payment_for_period: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.no_payment_for_period, Unset):
            no_payment_for_period = self.no_payment_for_period.to_dict()

        final_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.final_submission, Unset):
            final_submission = self.final_submission.to_dict()

        recoverable_amounts: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.recoverable_amounts, Unset):
            recoverable_amounts = self.recoverable_amounts.to_dict()

        apprenticeship_levy: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.apprenticeship_levy, Unset):
            apprenticeship_levy = self.apprenticeship_levy.to_dict()

        account: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account, Unset):
            account = self.account.to_dict()

        eligible_for_employment_allowance = self.eligible_for_employment_allowance
        de_minimis_state_aid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.de_minimis_state_aid, Unset):
            de_minimis_state_aid = self.de_minimis_state_aid.to_dict()

        i_rmark = self.i_rmark
        xml = self.xml
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        employer_references: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employer_references, Unset):
            employer_references = self.employer_references.to_dict()

        gov_talk_submission: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gov_talk_submission, Unset):
            gov_talk_submission = self.gov_talk_submission.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if period_of_inactivity is not UNSET:
            field_dict["periodOfInactivity"] = period_of_inactivity
        if no_payment_for_period is not UNSET:
            field_dict["noPaymentForPeriod"] = no_payment_for_period
        if final_submission is not UNSET:
            field_dict["finalSubmission"] = final_submission
        if recoverable_amounts is not UNSET:
            field_dict["recoverableAmounts"] = recoverable_amounts
        if apprenticeship_levy is not UNSET:
            field_dict["apprenticeshipLevy"] = apprenticeship_levy
        if account is not UNSET:
            field_dict["account"] = account
        if eligible_for_employment_allowance is not UNSET:
            field_dict[
                "eligibleForEmploymentAllowance"
            ] = eligible_for_employment_allowance
        if de_minimis_state_aid is not UNSET:
            field_dict["deMinimisStateAid"] = de_minimis_state_aid
        if i_rmark is not UNSET:
            field_dict["iRmark"] = i_rmark
        if xml is not UNSET:
            field_dict["xml"] = xml
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if employer_references is not UNSET:
            field_dict["employerReferences"] = employer_references
        if gov_talk_submission is not UNSET:
            field_dict["govTalkSubmission"] = gov_talk_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _period_of_inactivity = d.pop("periodOfInactivity", UNSET)
        period_of_inactivity: Union[Unset, FromToDates]
        if isinstance(_period_of_inactivity, Unset):
            period_of_inactivity = UNSET
        else:
            period_of_inactivity = FromToDates.from_dict(_period_of_inactivity)

        _no_payment_for_period = d.pop("noPaymentForPeriod", UNSET)
        no_payment_for_period: Union[Unset, FromToDates]
        if isinstance(_no_payment_for_period, Unset):
            no_payment_for_period = UNSET
        else:
            no_payment_for_period = FromToDates.from_dict(_no_payment_for_period)

        _final_submission = d.pop("finalSubmission", UNSET)
        final_submission: Union[Unset, EpsFinalSubmission]
        if isinstance(_final_submission, Unset):
            final_submission = UNSET
        else:
            final_submission = EpsFinalSubmission.from_dict(_final_submission)

        _recoverable_amounts = d.pop("recoverableAmounts", UNSET)
        recoverable_amounts: Union[Unset, RecoverableAmounts]
        if isinstance(_recoverable_amounts, Unset):
            recoverable_amounts = UNSET
        else:
            recoverable_amounts = RecoverableAmounts.from_dict(_recoverable_amounts)

        _apprenticeship_levy = d.pop("apprenticeshipLevy", UNSET)
        apprenticeship_levy: Union[Unset, EpsApprenticeshipLevy]
        if isinstance(_apprenticeship_levy, Unset):
            apprenticeship_levy = UNSET
        else:
            apprenticeship_levy = EpsApprenticeshipLevy.from_dict(_apprenticeship_levy)

        _account = d.pop("account", UNSET)
        account: Union[Unset, EpsAccount]
        if isinstance(_account, Unset):
            account = UNSET
        else:
            account = EpsAccount.from_dict(_account)

        eligible_for_employment_allowance = d.pop(
            "eligibleForEmploymentAllowance", UNSET
        )

        _de_minimis_state_aid = d.pop("deMinimisStateAid", UNSET)
        de_minimis_state_aid: Union[Unset, EpsDeMinimisStateAid]
        if isinstance(_de_minimis_state_aid, Unset):
            de_minimis_state_aid = UNSET
        else:
            de_minimis_state_aid = EpsDeMinimisStateAid.from_dict(_de_minimis_state_aid)

        i_rmark = d.pop("iRmark", UNSET)

        xml = d.pop("xml", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _employer_references = d.pop("employerReferences", UNSET)
        employer_references: Union[Unset, EmpRefs]
        if isinstance(_employer_references, Unset):
            employer_references = UNSET
        else:
            employer_references = EmpRefs.from_dict(_employer_references)

        _gov_talk_submission = d.pop("govTalkSubmission", UNSET)
        gov_talk_submission: Union[Unset, GovTalkSubmission]
        if isinstance(_gov_talk_submission, Unset):
            gov_talk_submission = UNSET
        else:
            gov_talk_submission = GovTalkSubmission.from_dict(_gov_talk_submission)

        id = d.pop("id", UNSET)

        eps = cls(
            period_of_inactivity=period_of_inactivity,
            no_payment_for_period=no_payment_for_period,
            final_submission=final_submission,
            recoverable_amounts=recoverable_amounts,
            apprenticeship_levy=apprenticeship_levy,
            account=account,
            eligible_for_employment_allowance=eligible_for_employment_allowance,
            de_minimis_state_aid=de_minimis_state_aid,
            i_rmark=i_rmark,
            xml=xml,
            tax_year=tax_year,
            employer_references=employer_references,
            gov_talk_submission=gov_talk_submission,
            id=id,
        )

        return eps



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.pay_run_state import PayRunState
from ..models.pay_run_state_change_reason import PayRunStateChangeReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="PayRunStateChange")


@attr.s(auto_attribs=True)
class PayRunStateChange:
    """
    Attributes:
        state (Union[Unset, PayRunState]): The state of the payrun. You would set this value when updating a payrun to
            finalise or re-open it.
            Other states are used with Bureau functionality which isn't currently generally available.
        reason (Union[Unset, PayRunStateChangeReason]):
        reason_text (Union[Unset, None, str]): A free-form text field for a reason for the change of state.
    """

    state: Union[Unset, PayRunState] = UNSET
    reason: Union[Unset, PayRunStateChangeReason] = UNSET
    reason_text: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        state: Union[Unset, str] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.value

        reason: Union[Unset, str] = UNSET
        if not isinstance(self.reason, Unset):
            reason = self.reason.value

        reason_text = self.reason_text

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if state is not UNSET:
            field_dict["state"] = state
        if reason is not UNSET:
            field_dict["reason"] = reason
        if reason_text is not UNSET:
            field_dict["reasonText"] = reason_text

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _state = d.pop("state", UNSET)
        state: Union[Unset, PayRunState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = PayRunState(_state)

        _reason = d.pop("reason", UNSET)
        reason: Union[Unset, PayRunStateChangeReason]
        if isinstance(_reason, Unset):
            reason = UNSET
        else:
            reason = PayRunStateChangeReason(_reason)

        reason_text = d.pop("reasonText", UNSET)

        pay_run_state_change = cls(
            state=state,
            reason=reason,
            reason_text=reason_text,
        )

        return pay_run_state_change



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.report import Report
from ..types import UNSET, Unset

T = TypeVar("T", bound="ReportPack")


@attr.s(auto_attribs=True)
class ReportPack:
    """A ReportPack is used to group together a number of reports

    Attributes:
        title (str): The name of this ReportPack
        reports (Union[Unset, None, List[Report]]): The Reports included in this ReportPack
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    title: str
    reports: Union[Unset, None, List[Report]] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        title = self.title
        reports: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.reports, Unset):
            if self.reports is None:
                reports = None
            else:
                reports = []
                for reports_item_data in self.reports:
                    reports_item = reports_item_data.value

                    reports.append(reports_item)

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "title": title,
            }
        )
        if reports is not UNSET:
            field_dict["reports"] = reports
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        title = d.pop("title")

        reports = []
        _reports = d.pop("reports", UNSET)
        for reports_item_data in _reports or []:
            reports_item = Report(reports_item_data)

            reports.append(reports_item)

        id = d.pop("id", UNSET)

        report_pack = cls(
            title=title,
            reports=reports,
            id=id,
        )

        return report_pack



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="UsageBill")


@attr.s(auto_attribs=True)
class UsageBill:
    """
    Attributes:
        year (Union[Unset, int]):
        month (Union[Unset, int]):
        payslip_count (Union[Unset, int]):
        previously_billed_payslip_count (Union[Unset, int]): The number of payslips that appear in the usage, but were
            billed in a previous period
        net_cost (Union[Unset, float]):
        discount (Union[Unset, float]):
        partner_discount_amount (Union[Unset, float]):
        monthly_minimum (Union[Unset, float]):
        total (Union[Unset, float]):
        paid (Union[Unset, bool]):
        usage (Union[Unset, None, List[Item]]):
        accounting_customer_id (Union[Unset, None, str]):
        accounting_invoice_id (Union[Unset, None, str]):
        accounting_invoice_link (Union[Unset, None, str]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    year: Union[Unset, int] = UNSET
    month: Union[Unset, int] = UNSET
    payslip_count: Union[Unset, int] = UNSET
    previously_billed_payslip_count: Union[Unset, int] = UNSET
    net_cost: Union[Unset, float] = UNSET
    discount: Union[Unset, float] = UNSET
    partner_discount_amount: Union[Unset, float] = UNSET
    monthly_minimum: Union[Unset, float] = UNSET
    total: Union[Unset, float] = UNSET
    paid: Union[Unset, bool] = UNSET
    usage: Union[Unset, None, List[Item]] = UNSET
    accounting_customer_id: Union[Unset, None, str] = UNSET
    accounting_invoice_id: Union[Unset, None, str] = UNSET
    accounting_invoice_link: Union[Unset, None, str] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        year = self.year
        month = self.month
        payslip_count = self.payslip_count
        previously_billed_payslip_count = self.previously_billed_payslip_count
        net_cost = self.net_cost
        discount = self.discount
        partner_discount_amount = self.partner_discount_amount
        monthly_minimum = self.monthly_minimum
        total = self.total
        paid = self.paid
        usage: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.usage, Unset):
            if self.usage is None:
                usage = None
            else:
                usage = []
                for usage_item_data in self.usage:
                    usage_item = usage_item_data.to_dict()

                    usage.append(usage_item)

        accounting_customer_id = self.accounting_customer_id
        accounting_invoice_id = self.accounting_invoice_id
        accounting_invoice_link = self.accounting_invoice_link
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if year is not UNSET:
            field_dict["year"] = year
        if month is not UNSET:
            field_dict["month"] = month
        if payslip_count is not UNSET:
            field_dict["payslipCount"] = payslip_count
        if previously_billed_payslip_count is not UNSET:
            field_dict["previouslyBilledPayslipCount"] = previously_billed_payslip_count
        if net_cost is not UNSET:
            field_dict["netCost"] = net_cost
        if discount is not UNSET:
            field_dict["discount"] = discount
        if partner_discount_amount is not UNSET:
            field_dict["partnerDiscountAmount"] = partner_discount_amount
        if monthly_minimum is not UNSET:
            field_dict["monthlyMinimum"] = monthly_minimum
        if total is not UNSET:
            field_dict["total"] = total
        if paid is not UNSET:
            field_dict["paid"] = paid
        if usage is not UNSET:
            field_dict["usage"] = usage
        if accounting_customer_id is not UNSET:
            field_dict["accountingCustomerId"] = accounting_customer_id
        if accounting_invoice_id is not UNSET:
            field_dict["accountingInvoiceId"] = accounting_invoice_id
        if accounting_invoice_link is not UNSET:
            field_dict["accountingInvoiceLink"] = accounting_invoice_link
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        year = d.pop("year", UNSET)

        month = d.pop("month", UNSET)

        payslip_count = d.pop("payslipCount", UNSET)

        previously_billed_payslip_count = d.pop("previouslyBilledPayslipCount", UNSET)

        net_cost = d.pop("netCost", UNSET)

        discount = d.pop("discount", UNSET)

        partner_discount_amount = d.pop("partnerDiscountAmount", UNSET)

        monthly_minimum = d.pop("monthlyMinimum", UNSET)

        total = d.pop("total", UNSET)

        paid = d.pop("paid", UNSET)

        usage = []
        _usage = d.pop("usage", UNSET)
        for usage_item_data in _usage or []:
            usage_item = Item.from_dict(usage_item_data)

            usage.append(usage_item)

        accounting_customer_id = d.pop("accountingCustomerId", UNSET)

        accounting_invoice_id = d.pop("accountingInvoiceId", UNSET)

        accounting_invoice_link = d.pop("accountingInvoiceLink", UNSET)

        id = d.pop("id", UNSET)

        usage_bill = cls(
            year=year,
            month=month,
            payslip_count=payslip_count,
            previously_billed_payslip_count=previously_billed_payslip_count,
            net_cost=net_cost,
            discount=discount,
            partner_discount_amount=partner_discount_amount,
            monthly_minimum=monthly_minimum,
            total=total,
            paid=paid,
            usage=usage,
            accounting_customer_id=accounting_customer_id,
            accounting_invoice_id=accounting_invoice_id,
            accounting_invoice_link=accounting_invoice_link,
            id=id,
        )

        return usage_bill



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="GrossToNetReportLine")


@attr.s(auto_attribs=True)
class GrossToNetReportLine:
    """
    Attributes:
        total_gross (Union[Unset, float]):
        taxable_gross (Union[Unset, float]):
        net_pay (Union[Unset, float]):
        tax (Union[Unset, float]):
        employee_ni (Union[Unset, float]):
        employer_ni (Union[Unset, float]):
        employee_pension (Union[Unset, float]):
        employer_pension (Union[Unset, float]):
        student_or_pg_loan (Union[Unset, float]):
        statutory_payments (Union[Unset, float]):
        attachments (Union[Unset, float]):
        other_deductions (Union[Unset, float]):
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        department (Union[Unset, None, str]):
    """

    total_gross: Union[Unset, float] = UNSET
    taxable_gross: Union[Unset, float] = UNSET
    net_pay: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    employee_ni: Union[Unset, float] = UNSET
    employer_ni: Union[Unset, float] = UNSET
    employee_pension: Union[Unset, float] = UNSET
    employer_pension: Union[Unset, float] = UNSET
    student_or_pg_loan: Union[Unset, float] = UNSET
    statutory_payments: Union[Unset, float] = UNSET
    attachments: Union[Unset, float] = UNSET
    other_deductions: Union[Unset, float] = UNSET
    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    department: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        total_gross = self.total_gross
        taxable_gross = self.taxable_gross
        net_pay = self.net_pay
        tax = self.tax
        employee_ni = self.employee_ni
        employer_ni = self.employer_ni
        employee_pension = self.employee_pension
        employer_pension = self.employer_pension
        student_or_pg_loan = self.student_or_pg_loan
        statutory_payments = self.statutory_payments
        attachments = self.attachments
        other_deductions = self.other_deductions
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        department = self.department

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if total_gross is not UNSET:
            field_dict["totalGross"] = total_gross
        if taxable_gross is not UNSET:
            field_dict["taxableGross"] = taxable_gross
        if net_pay is not UNSET:
            field_dict["netPay"] = net_pay
        if tax is not UNSET:
            field_dict["tax"] = tax
        if employee_ni is not UNSET:
            field_dict["employeeNi"] = employee_ni
        if employer_ni is not UNSET:
            field_dict["employerNi"] = employer_ni
        if employee_pension is not UNSET:
            field_dict["employeePension"] = employee_pension
        if employer_pension is not UNSET:
            field_dict["employerPension"] = employer_pension
        if student_or_pg_loan is not UNSET:
            field_dict["studentOrPgLoan"] = student_or_pg_loan
        if statutory_payments is not UNSET:
            field_dict["statutoryPayments"] = statutory_payments
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if other_deductions is not UNSET:
            field_dict["otherDeductions"] = other_deductions
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if department is not UNSET:
            field_dict["department"] = department

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total_gross = d.pop("totalGross", UNSET)

        taxable_gross = d.pop("taxableGross", UNSET)

        net_pay = d.pop("netPay", UNSET)

        tax = d.pop("tax", UNSET)

        employee_ni = d.pop("employeeNi", UNSET)

        employer_ni = d.pop("employerNi", UNSET)

        employee_pension = d.pop("employeePension", UNSET)

        employer_pension = d.pop("employerPension", UNSET)

        student_or_pg_loan = d.pop("studentOrPgLoan", UNSET)

        statutory_payments = d.pop("statutoryPayments", UNSET)

        attachments = d.pop("attachments", UNSET)

        other_deductions = d.pop("otherDeductions", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        department = d.pop("department", UNSET)

        gross_to_net_report_line = cls(
            total_gross=total_gross,
            taxable_gross=taxable_gross,
            net_pay=net_pay,
            tax=tax,
            employee_ni=employee_ni,
            employer_ni=employer_ni,
            employee_pension=employee_pension,
            employer_pension=employer_pension,
            student_or_pg_loan=student_or_pg_loan,
            statutory_payments=statutory_payments,
            attachments=attachments,
            other_deductions=other_deductions,
            employee=employee,
            payroll_code=payroll_code,
            department=department,
        )

        return gross_to_net_report_line
